from datetime import date

import SpectrumCommon.Const.GB as gbc
import SpectrumCommon.Modvars.GB.GBUtil as gbu

import SpectrumCommon.Const.PJ.PJNTags as pjmvt

import SpectrumCommon.Const.PC.PCConst as pcc
import SpectrumCommon.Const.PC.PCConstLink as pccl
import SpectrumCommon.Const.PC.PCTags as pcmvt
import SpectrumCommon.Modvars.PC.PCModVarDefs as pcmvd
import SpectrumCommon.Modvars.PC.PCModVarUtil as pcmvu

import SpectrumCommon.Modvars.PC.PCReadDatabases as pcrd

def init_modvars(country_ID, first_year, final_year, extra):
    '''
        Initializes Programme Costing MVs for new IHT projections.

            Parameters:
                countryID (string):  Ex: BEN for Benin.

                first_year (int): First year selected by user.

                final_year (int): Final year selected by user. 

                extra (dict): Any other fields.  
 
            Returns:
                mod_vars_rec (dict): MVs.
    '''
    
    modules = extra[gbc.GB_MODULES]

    IHT_costing_on = extra[gbc.GB_IHT_ON] if gbc.GB_IHT_ON in extra else False
    TB_costing_on = extra[gbc.GB_TB_COSTING_ON] if gbc.GB_TB_COSTING_ON in extra else False
    LiST_costing_on = extra[gbc.GB_LIST_COSTING_ON] if gbc.GB_LIST_COSTING_ON in extra else False
    costing_mode_mod_vars = {
        pjmvt.GB_IHTOnTag       : IHT_costing_on,
        pjmvt.GB_TBCostingOnTag : TB_costing_on,
        pjmvt.GB_LCOnTag        : LiST_costing_on,
    }

    costing_mode = gbu.get_costing_mode(costing_mode_mod_vars)

    # health_sys_admin_unit_records = extra[pccl.PC_IH_HEALTH_SYS_ADMIN_UNIT_RECORDS_TAG]

    # Load databases
    cost_cat_DB = pcrd.PC_load_cost_cat_DB()
    cost_input_DB = pcrd.PC_load_cost_input_DB()
    costing_sect_DB = pcrd.PC_load_costing_sect_DB()
    activity_DB = pcrd.PC_load_activity_DB()
    activity_line_item_DB = pcrd.PC_load_activity_line_item_DB()
    PPLI_activity_line_item_DB = pcrd.PC_load_PPLI_activity_line_item_DB()
    PPLI_phase_DB = pcrd.PC_load_PPLI_phase_DB(country_ID)
    PPLI_taxation_DB = pcrd.PC_load_PPLI_taxation_DB(country_ID)

    value_bool = False
    value_int = 0
    value_flt = 0.0
    value_str = ''
    value_date = date.today().strftime("%B %d, %Y"),   

    num_years_inputs = final_year - first_year + 1

    #prog_area_ID_list = pcrd.get_prog_area_ID_list(group_DB)
    #area_data_list = pcmvd.init_area_dict_list(prog_area_ID_list, num_total_years)
    area_data_list = pcmvd.PC_init_area_dict_list(pccl.PC_IH_PROG_AREA_MST_ID, extra[pccl.PC_IH_PROG_AREA_RECORDS_TAG_V1], num_years_inputs, costing_mode)
    
    cost_cat_records = pcrd.PC_init_cost_cat_records(cost_cat_DB)
    cost_input_records = pcrd.PC_init_cost_input_records(cost_input_DB)
    costing_sect_records = pcrd.PC_init_costing_sect_records(costing_sect_DB, num_years_inputs, costing_mode, modules, False)
    activity_records = pcrd.PC_init_activity_records(activity_DB, costing_mode, modules, False)
    activity_line_item_records = pcrd.PC_init_activity_line_item_records(activity_line_item_DB, num_years_inputs, costing_mode, modules)

    costing_sect_records_PPLI = []
    activity_records_PPLI = []
    activity_line_item_records_PPLI = []
    if costing_mode == gbc.GB_IHT_COSTING_MODE:
        costing_sect_records_PPLI = pcrd.PC_init_costing_sect_records(costing_sect_DB, num_years_inputs, costing_mode, modules, True)
        activity_records_PPLI = pcrd.PC_init_activity_records(activity_DB, costing_mode, modules, True)
        # The activity line item database requires the phases to be set first.
        pcrd.PC_init_PPLI_phase_and_intensity(PPLI_phase_DB, costing_sect_records_PPLI, num_years_inputs)
        activity_line_item_records_PPLI = pcrd.PC_init_PPLI_activity_line_item_records(PPLI_activity_line_item_DB, costing_sect_records_PPLI, num_years_inputs)
        tobacco_costing_sect_rec = pcmvu.get_PC_costing_sect_rec(costing_sect_records_PPLI, pccl.PC_IH_PPLI_MST_ID, str(pccl.PC_IH_NCD_MST_ID), str(pcc.PC_PPLI_TOBACCO_CTRL_COST_SECT_ID))
        pcrd.PC_set_price_elasticity_params(PPLI_taxation_DB, tobacco_costing_sect_rec)

    # deliv_chan_pack_record_list []
    # nat_prog_record_list = []
    # audit_record_list = []
    # locked_area_record_list = []

    # add flag modvars to one modes modvar. If flag is in set, turn mode on. Else, turn mode off. Just like how you will do with intervention sets in IC.

    mod_vars_rec = {            
        pcmvt.PC_VERSION_TAG               : 1,
        pcmvt.PC_DATA_CHANGED_TAG          : value_bool, 
        pcmvt.PC_PROJ_VALID_TAG            : value_bool, 
        pcmvt.PC_FILE_NAME_TAG             : value_str,                          
        pcmvt.PC_PROJ_VALID_DATE_TAG       : value_date,    

        pcmvt.PC_AREA_DATA_TAG             : area_data_list,
        
        pcmvt.PC_COST_CAT_RECORDS_TAG      : cost_cat_records,
        pcmvt.PC_COSTING_SECT_RECORDS_TAG  : costing_sect_records, 
        pcmvt.PC_COST_INPUT_RECORDS_TAG    : cost_input_records,
        pcmvt.PC_ACT_RECORDS_TAG           : activity_records, 
        pcmvt.PC_ACT_LINE_ITEM_RECORDS_TAG : activity_line_item_records,

        pcmvt.PC_COSTING_SECT_RECORDS_PPLI_TAG  : costing_sect_records_PPLI,
        pcmvt.PC_ACT_RECORDS_PPLI_TAG           : activity_records_PPLI,
        pcmvt.PC_ACT_LINE_ITEM_RECORDS_PPLI_TAG : activity_line_item_records_PPLI,

        pcmvt.PC_RES_COSTS_BY_SECT_TAG    : [],
        pcmvt.PC_RES_COSTS_BY_SUBSECT_TAG : [],
        pcmvt.PC_RES_COSTS_BY_ACT_TAG     : [],
        pcmvt.PC_RES_COSTS_BY_CAT_TAG     : [],

        pcmvt.PC_RES_COSTS_BY_SECT_PPLI_TAG    : [],
        pcmvt.PC_RES_COSTS_BY_SUBSECT_PPLI_TAG : [],
        pcmvt.PC_RES_COSTS_BY_ACT_PPLI_TAG     : [],
        pcmvt.PC_RES_COSTS_BY_CAT_PPLI_TAG     : [],

        pcmvt.PC_STD_ROW_INFO_META_TAG : pcmvd.PC_init_row_info_meta(gbc.GB_NOT_FOUND), # Standard
        pcmvt.PC_HIV_ROW_INFO_META_TAG : pcmvd.PC_init_row_info_meta(pccl.PC_IH_HIV_AIDS_MST_ID) # HIV
    }

    if costing_mode == gbc.GB_LIST_COSTING_MODE:
        mod_vars_rec[pcmvt.PC_ENTER_COSTING_SECT_COSTS_ALL_YEARS_CS_TAG] = False

    #Debugging
    #with open('C:\Proj\IHTSampleFile.JSON', 'w') as f:
    #   json.dump(mod_vars_dict, f)

    return mod_vars_rec

    


# Contains methods for setting up OHT ModVars

from os import environ

from AvenirCommon.Database import GB_get_db_json

import SpectrumCommon.Const.GB as gbc

import SpectrumCommon.Const.BG.BGConst as bgc
import SpectrumCommon.Const.BG.BGConstLink as bgcl
import SpectrumCommon.Const.BG.BGDatabaseKeys as bgdbk
import SpectrumCommon.Const.BG.BGTags as bgmvt
import SpectrumCommon.Modvars.BG.BGModVarUtil as bgmvu
import SpectrumCommon.Modvars.BG.BGModVarDefs as bgmvd
import SpectrumCommon.Util.BG.BGUtilLink as bgul

#######################################################################################################
#                                                                                                     #
#                                       Database loading methods                                      #
#                                                                                                     #
#######################################################################################################

#########################################   Inputs   ######################################################

def BG_load_budget_DB():
    budget_DB_file_name = bgc.BG_BUDGET_DB_NAME + '_' + bgc.BG_BUDGET_DB_CURR_VERSION + '.' + gbc.GB_JSON
    budget_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_BG_CONTAINER, budget_DB_file_name)

    return budget_DB

def BG_load_budget_category_DB():
    budget_category_DB_file_name = bgc.BG_BUDGET_CAT_DB_NAME + '_' + bgc.BG_BUDGET_CAT_DB_CURR_VERSION + '.' + gbc.GB_JSON
    budget_category_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_BG_CONTAINER, budget_category_DB_file_name)

    return budget_category_DB

def BG_load_indicator_mapping_DB():
    ind_mapping_DB_file_name = bgc.BG_INDICATOR_MAPPING_DB_NAME + '_' + bgc.BG_INDICATOR_MAPPING_DB_CURR_VERSION + '.' + gbc.GB_JSON
    ind_mapping_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_BG_CONTAINER, ind_mapping_DB_file_name)

    return ind_mapping_DB

def BG_load_funding_framework_DB():
    funding_framework_DB_file_name = bgc.BG_FUND_FW_DB_NAME + '_' + bgc.BG_FUND_FW_DB_CURR_VERSION + '.' + gbc.GB_JSON
    funding_framework_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_BG_CONTAINER, funding_framework_DB_file_name)

    return funding_framework_DB

def BG_load_funding_source_DB():
    funding_source_DB_file_name = bgc.BG_FUND_SOURCE_DB_NAME + '_' + bgc.BG_FUND_SOURCE_DB_CURR_VERSION + '.' + gbc.GB_JSON
    funding_source_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_BG_CONTAINER, funding_source_DB_file_name)

    return funding_source_DB

#######################################################################################################
#                                                                                                     #
#                                       Budgets                                                       #
#                                                                                                     #
#######################################################################################################

def BG_init_budget_records(budget_DB, costing_mode, num_dynamic_staff_types = int, budget_ID = ''):
    budget_records = []

    for budget_DB_obj in budget_DB:
        if (budget_ID == '') or (budget_ID == str(budget_DB_obj[bgdbk.BG_ID_KEY_BDB])):
            if (((costing_mode == gbc.GB_IHT_COSTING_MODE) and (str(gbc.GB_IHT_COSTING_MODE) in budget_DB_obj[bgdbk.BG_COSTING_MODES_KEY_BDB])) or
                ((costing_mode == gbc.GB_TB_COSTING_MODE) and (str(gbc.GB_TB_COSTING_MODE) in budget_DB_obj[bgdbk.BG_COSTING_MODES_KEY_BDB]))):
                    budget_records.append(BG_init_budget_rec(budget_DB_obj, num_dynamic_staff_types))

    return budget_records

def BG_init_budget_rec(budget_DB_obj = dict, num_dynamic_staff_types = int):

    ID = str(budget_DB_obj[bgdbk.BG_ID_KEY_BDB]) 
    english_str = budget_DB_obj[bgdbk.BG_ENGLISH_STRING_KEY_BDB] 
    str_constant = budget_DB_obj[bgdbk.BG_STRING_CONSTANT_KEY_BDB] 

    budget_rec = bgmvd.BG_create_budget_rec(ID, english_str, str_constant, num_dynamic_staff_types)

    return budget_rec

#######################################################################################################
#                                                                                                     #
#                                       Budget categories                                             #
#                                                                                                     #
#######################################################################################################

def BG_init_budget_cat_records(budget_cat_DB, costing_mode, num_years = int, budget_ID = ''):
    budget_cat_records = []

    for budget_cat_DB_obj in budget_cat_DB:
        if (budget_ID == '') or (budget_ID == str(budget_cat_DB_obj[bgdbk.BG_BUDGET_ID_KEY_BCDB])):
            if (((costing_mode == gbc.GB_IHT_COSTING_MODE) and (str(gbc.GB_IHT_COSTING_MODE) in budget_cat_DB_obj[bgdbk.BG_COSTING_MODES_KEY_BCDB])) or
                ((costing_mode == gbc.GB_TB_COSTING_MODE) and (str(gbc.GB_TB_COSTING_MODE) in budget_cat_DB_obj[bgdbk.BG_COSTING_MODES_KEY_BCDB]))):
                    budget_cat_records.append(BG_init_budget_cat_rec(budget_cat_DB_obj, num_years))

    return budget_cat_records

def BG_init_budget_cat_rec(budget_cat_DB_obj, num_years = int):

    budget_ID = str(budget_cat_DB_obj[bgdbk.BG_BUDGET_ID_KEY_BCDB]) 
    budget_cat_ID = str(budget_cat_DB_obj[bgdbk.BG_ID_KEY_BCDB]) 
    english_str = budget_cat_DB_obj[bgdbk.BG_ENGLISH_STRING_KEY_BCDB] 
    str_constant = budget_cat_DB_obj[bgdbk.BG_STRING_CONSTANT_KEY_BCDB] 

    budget_cat_rec = bgmvd.BG_create_budget_category_rec(budget_ID, budget_cat_ID, english_str, str_constant, num_years)

    return budget_cat_rec

#######################################################################################################
#                                                                                                     #
#                                       Indicator mappings                                            #
#                                                                                                     #
#######################################################################################################

def BG_set_default_indicator_mappings(ind_mapping_DB, required_mod_vars, costing_mode, budget_ID):
    '''
        This method will initialize everything related to budgets rather than just the budget records themselves.

            Parameters:
                ind_mapping_DB (dict): Database of budget category mappings for all indicators.

                required_mod_vars (dict): MV tag, MV value pairs of all MVs required to execute this function.  The following MVs are required:

                    BG_INDICATOR_RECORDS_IC_TAG  
                    BG_INDICATOR_RECORDS_PC_TAG        
                    BG_INDICATOR_RECORDS_IS_TAG   

                    IH_DELIV_CHAN_RECORDS_TAG_V1
                    IH_PROG_AREA_RECORDS_TAG_V1

                budget_ID (str) : budget ID.
              
            Returns:
                Nothing.
    '''

    ind_records_IC_mv = required_mod_vars[bgmvt.BG_INDICATOR_RECORDS_IC_TAG]
    ind_records_PC_mv = required_mod_vars[bgmvt.BG_INDICATOR_RECORDS_PC_TAG]
    ind_records_IS_mv = required_mod_vars[bgmvt.BG_INDICATOR_RECORDS_IS_TAG]

    deliv_chan_records_mv = required_mod_vars[bgcl.BG_IH_DELIV_CHAN_RECORDS_TAG]
    prog_area_records_mv = required_mod_vars[bgcl.BG_IH_PROG_AREA_RECORDS_TAG]

    def map_PC_rec(ind_mapping_obj_l, prog_area_ID):
        ind_rec = bgmvu.get_BG_ind_record_PC(
            ind_records_PC_mv, budget_ID, int(ind_mapping_obj_l[bgdbk.BG_IND_TYPE_IDS_KEY_IMDB][0]), 
            prog_area_ID,
            str(ind_mapping_obj_l[bgdbk.BG_COSTING_SECT_ID_KEY_IMDB]), 
            str(ind_mapping_obj_l[bgdbk.BG_COSTING_SUBSECT_ID_KEY_IMDB]), 
            str(ind_mapping_obj_l[bgdbk.BG_ACTIVITY_ID_KEY_IMDB])
        )

        if ind_rec != gbc.GB_NOT_FOUND: 
            bgmvu.set_BG_ind_budget_cat_ID(ind_rec, str(ind_mapping_obj_l[bgdbk.BG_BUDGET_CAT_ID_KEY_IMDB]))

    for ind_mapping_obj in ind_mapping_DB:

        if (budget_ID == '') or (budget_ID == str(ind_mapping_obj[bgdbk.BG_BUDGET_ID_KEY_IMDB])):

            if (((costing_mode == gbc.GB_IHT_COSTING_MODE) and (str(gbc.GB_IHT_COSTING_MODE) in ind_mapping_obj[bgdbk.BG_COSTING_MODES_KEY_IMDB])) or
                ((costing_mode == gbc.GB_TB_COSTING_MODE) and (str(gbc.GB_TB_COSTING_MODE) in ind_mapping_obj[bgdbk.BG_COSTING_MODES_KEY_IMDB]))):
                    
                    if ind_mapping_obj[bgdbk.BG_MOD_ID_KEY_IMDB] == gbc.GB_PC:
                        if costing_mode == gbc.GB_IHT_COSTING_MODE:
                            for prog_area_rec in prog_area_records_mv:
                                prog_area_ID = bgul.BG_get_IH_prog_area_ID(prog_area_rec)

                                map_PC_rec(ind_mapping_obj, prog_area_ID)

                        elif costing_mode == gbc.GB_TB_COSTING_MODE:
                             
                            map_PC_rec(ind_mapping_obj, str(gbc.GB_NOT_FOUND))

                    elif ind_mapping_obj[bgdbk.BG_MOD_ID_KEY_IMDB] == gbc.GB_IC:
                        # Database currently assumes all values are repeated for all delivery channels.
                        for deliv_chan_rec in deliv_chan_records_mv:
                            deliv_chan_ID = bgul.BG_get_IH_deliv_chan_ID(deliv_chan_rec)

                            for ind_type_ID in ind_mapping_obj[bgdbk.BG_IND_TYPE_IDS_KEY_IMDB]:
                                ind_rec = bgmvu.get_BG_ind_record_IC_deliv_chan(ind_records_IC_mv, budget_ID, int(ind_type_ID), str(ind_mapping_obj[bgdbk.BG_IND_ID_KEY_IMDB]), False, deliv_chan_ID)

                                if ind_rec != gbc.GB_NOT_FOUND: 
                                    bgmvu.set_BG_ind_budget_cat_ID(ind_rec, str(ind_mapping_obj[bgdbk.BG_BUDGET_CAT_ID_KEY_IMDB]))

                        # All delivery channels combined.
                        for ind_type_ID in ind_mapping_obj[bgdbk.BG_IND_TYPE_IDS_KEY_IMDB]:
                            ind_rec = bgmvu.get_BG_ind_record_IC_deliv_chan(ind_records_IC_mv, budget_ID, int(ind_type_ID), str(ind_mapping_obj[bgdbk.BG_IND_ID_KEY_IMDB]), False, str(gbc.GB_NOT_FOUND))
                            
                            if ind_rec != gbc.GB_NOT_FOUND: 
                                bgmvu.set_BG_ind_budget_cat_ID(ind_rec, str(ind_mapping_obj[bgdbk.BG_BUDGET_CAT_ID_KEY_IMDB]))

                    elif ind_mapping_obj[bgdbk.BG_MOD_ID_KEY_IMDB] == gbc.GB_IS:
                        # Currently, only TB Costing mode has default budget category mappings, so we can use "-1" for the prog area.
                        ind_rec = bgmvu.get_BG_ind_record_IS_non_prog_mgmt(
                            ind_records_IS_mv, budget_ID, int(ind_mapping_obj[bgdbk.BG_IND_TYPE_IDS_KEY_IMDB][0]), str(ind_mapping_obj[bgdbk.BG_IND_ID_KEY_IMDB]), str(gbc.GB_NOT_FOUND)
                        )

                        if ind_rec != gbc.GB_NOT_FOUND: 
                            bgmvu.set_BG_ind_budget_cat_ID(ind_rec, str(ind_mapping_obj[bgdbk.BG_BUDGET_CAT_ID_KEY_IMDB]))

#######################################################################################################
#                                                                                                     #
#                                       Funding frameworks                                            #
#                                                                                                     #
#######################################################################################################

def BG_init_fund_framework_records(fund_framework_DB, costing_mode, fund_framework_ID = ''):
    fund_framework_records = []

    for fund_framework_DB_obj in fund_framework_DB:
        if (fund_framework_ID == '') or (fund_framework_ID == str(fund_framework_DB_obj[bgdbk.BG_ID_KEY_FFWDB])):
            if (((costing_mode == gbc.GB_IHT_COSTING_MODE) and (str(gbc.GB_IHT_COSTING_MODE) in fund_framework_DB_obj[bgdbk.BG_COSTING_MODES_KEY_FFWDB])) or
                ((costing_mode == gbc.GB_TB_COSTING_MODE) and (str(gbc.GB_TB_COSTING_MODE) in fund_framework_DB_obj[bgdbk.BG_COSTING_MODES_KEY_FFWDB]))):
                    fund_framework_records.append(BG_init_fund_framework_rec(fund_framework_DB_obj))

    return fund_framework_records

def BG_init_fund_framework_rec(fund_framework_DB_obj):

    ID = str(fund_framework_DB_obj[bgdbk.BG_ID_KEY_FFWDB]) 
    english_str = fund_framework_DB_obj[bgdbk.BG_ENGLISH_STRING_KEY_FFWDB] 
    str_constant = fund_framework_DB_obj[bgdbk.BG_STRING_CONSTANT_KEY_FFWDB] 

    fund_framework_rec = bgmvd.BG_create_funding_framework_rec(ID, english_str, str_constant)

    return fund_framework_rec

#######################################################################################################
#                                                                                                     #
#                                       Funding sources                                               #
#                                                                                                     #
#######################################################################################################

def BG_init_fund_source_records(fund_source_DB, costing_mode, fund_framework_ID = ''):
    fund_source_records = []

    for fund_source_DB_obj in fund_source_DB:
        if (fund_framework_ID == '') or (fund_framework_ID == str(fund_source_DB_obj[bgdbk.BG_FUND_FW_ID_KEY_FSDB])):
            if (((costing_mode == gbc.GB_IHT_COSTING_MODE) and (str(gbc.GB_IHT_COSTING_MODE) in fund_source_DB_obj[bgdbk.BG_COSTING_MODES_KEY_FSDB])) or
                ((costing_mode == gbc.GB_TB_COSTING_MODE) and (str(gbc.GB_TB_COSTING_MODE) in fund_source_DB_obj[bgdbk.BG_COSTING_MODES_KEY_FSDB]))):
                    fund_source_records.append(BG_init_fund_source_rec(fund_source_DB_obj))

    return fund_source_records

def BG_init_fund_source_rec(fund_source_DB_obj):

    fund_framework_ID = str(fund_source_DB_obj[bgdbk.BG_FUND_FW_ID_KEY_FSDB]) 
    fund_source_ID = str(fund_source_DB_obj[bgdbk.BG_ID_KEY_FSDB]) 
    english_str = fund_source_DB_obj[bgdbk.BG_ENGLISH_STRING_KEY_FSDB] 
    str_constant = fund_source_DB_obj[bgdbk.BG_STRING_CONSTANT_KEY_FSDB] 

    found_source_rec = bgmvd.BG_create_funding_source_rec(fund_framework_ID, fund_source_ID, english_str, str_constant)

    return found_source_rec

########################################################################################################################################
########################################################################################################################################
#                                                                                                                                      #
#                                                       Meta data                                                                      #
#                                                                                                                                      #
########################################################################################################################################
########################################################################################################################################

def BG_init_default_budget_list_meta(budget_DB, costing_mode):
    default_budget_list_meta = []

    for budget_DB_obj in budget_DB:
        if (((costing_mode == gbc.GB_IHT_COSTING_MODE) and (str(gbc.GB_IHT_COSTING_MODE) in budget_DB_obj[bgdbk.BG_COSTING_MODES_KEY_BDB])) or
            ((costing_mode == gbc.GB_TB_COSTING_MODE) and (str(gbc.GB_TB_COSTING_MODE) in budget_DB_obj[bgdbk.BG_COSTING_MODES_KEY_BDB]))):
                default_budget_list_meta.append({
                    'mstID'          : str(budget_DB_obj[bgdbk.BG_ID_KEY_BDB]),
                    'name'           : budget_DB_obj[bgdbk.BG_ENGLISH_STRING_KEY_BDB],
                    'stringConstant' : budget_DB_obj[bgdbk.BG_STRING_CONSTANT_KEY_BDB]
                })
    
    return default_budget_list_meta

def BG_init_default_fund_framework_list_meta(fund_fw_DB, costing_mode):
    default_fund_fw_list_meta = []

    for fund_fw_DB_obj in fund_fw_DB:
        if (((costing_mode == gbc.GB_IHT_COSTING_MODE) and (str(gbc.GB_IHT_COSTING_MODE) in fund_fw_DB_obj[bgdbk.BG_COSTING_MODES_KEY_FFWDB])) or
            ((costing_mode == gbc.GB_TB_COSTING_MODE) and (str(gbc.GB_TB_COSTING_MODE) in fund_fw_DB_obj[bgdbk.BG_COSTING_MODES_KEY_FFWDB]))):
                default_fund_fw_list_meta.append({
                    'mstID'          : str(fund_fw_DB_obj[bgdbk.BG_ID_KEY_FFWDB]),
                    'name'           : fund_fw_DB_obj[bgdbk.BG_ENGLISH_STRING_KEY_FFWDB],
                    'stringConstant' : fund_fw_DB_obj[bgdbk.BG_STRING_CONSTANT_KEY_FFWDB]
                })
    
    return default_fund_fw_list_meta




from SpectrumCommon.Const.HW import *
import SpectrumCommon.Const.HW.HWTags as hwmvt
from SpectrumCommon.Modvars.GB.GBDefs import *

hw_staff_dim = 'hw_staff_dim'
hw_staff_id = {
}

hw_institutions_dim = 'hw_institutions_dim'
hw_institution_id = {
    
}
hw_attrition_type_dim = 'hw_attrition_type'
hw_attrition_type_ids = {}

hw_retention_type_dim ='hw_retention_type'
hw_retention_type_ids = {}

hw_staffing_levels_dim = 'hw_staffing_levels_dim'
hw_staffing_levels_id = {
        HW_STAFF_LEVEL_FACILITY_CURR_ID : 'Facility',
        HW_STAFF_LEVEL_DISTRICT_CURR_ID : 'District',
        HW_STAFF_LEVEL_REGIONAL_CURR_ID : 'Regional',
        HW_STAFF_LEVEL_NATIONAL_CURR_ID : 'National',
}
hw_staff_types_dim = 'hw_staff_types'
hw_staff_types_id = {
    
}

hw_staff_comp_dim = 'hw_staff_comp'
hw_staff_comp_id = {}

hw_facility_types_dim = 'hw_facility_types_dim'
hw_facility_types_id = {
}

hw_inservice_dim = 'hw_inservice_dim'    
hw_inservice_id = {}

hw_staff_trained_dim = 'hw_staff_trained'
hw_staff_trained_id = {}

hw_pc_standard_rows_dim = 'hw_pc_standard_rows_dim'
hw_pc_standard_rows_id = {}
    
hw_static_staff_dim = 'hw_static_staff_dim'
hw_static_staff_id = {
        HW_GENERAL_MEDICAL_PRACTITIONERS_CURR_ID           : 'General medical practitioners',        
        HW_SPECIAL_MEDICAL_PRACTITIONERS_CURR_ID           : 'Special medical practitioners',        
        HW_CLINIC_OFF_SURG_TECHS_CURR_ID                   : 'Clinic off surg techs',        
        HW_NURSING_PROFESSIONALS_CURR_ID                   : 'Nursing professionals',        
        HW_MIDWIFERY_PROFESSIONALS_CURR_ID                 : 'Midwifery professionals',        
        HW_MIDWIFERY_ASSOCIATE_PROFESSIONALS_CURR_ID       : 'Midwifery associate professionals',        
        HW_NURSING_ASSOCIATE_PROFESSIONALS_CURR_ID         : 'Nursing associate professionals',        
        HW_MED_PATH_LAB_TECHS_CURR_ID                      : 'Med path lab techs',       
        HW_PHARM_TECHS_ASSIST_CURR_ID                      : 'Pharm techs assist',        
        HW_MED_IMAGING_THERA_EQUIP_TECHS_CURR_ID           : 'Med imaging thera equip techs',
        HW_AMBULANCE_WORKERS_CURR_ID                       : 'Ambulance workers',
        HW_COMM_HLTH_WORKERS_CURR_ID                       : 'Comm hlth workers',
        HW_OTHER_STAFF_TYPE_CURR_ID                        : 'Other staff type',
        HW_DENTISTS_CURR_ID                                : 'Dentists',
        HW_DENTAL_ASSIST_THERAPISTS_CURR_ID                : 'Dental assist therapists',
        HW_MED_DENTAL_PROSTHETIC_REL_TECHS_CURR_ID         : 'Med dental prosthetic rel techs',
        HW_PHARMACISTS_CURR_ID                             : 'Pharmacists',
        HW_PARAMED_PRACTITIONERS_CURR_ID                   : 'Paramed practitioners',
        HW_ENVIRON_OCCUPAT_HEALTH_HYGIENE_PROF_CURR_ID     : 'Environ occupat health hygiene prof',
        HW_DIETICIANS_NUTRITIONISTS_CURR_ID                : 'Dieticians nutritionists',
        HW_AUDIOLOGISTS_SPEECH_THERAPISTS_CURR_ID          : 'Audiologists speech therapists',
        HW_OPTOMETRISTS_OPHTHALMIC_OPTICIANS_CURR_ID       : 'Optometrists ophthalmic opticians',
        HW_DISPENSING_OPTICIANS_CURR_ID                    : 'Dispensing opticians',
        HW_ENVIRON_OCCUPAT_HEALTH_INSPEC_ASSO_CURR_IDC     : 'Environ occupat health inspec assoc',
        HW_TRAD_COMP_MED_PROF_CURR_ID                      : 'Trad comp med prof',
        HW_TRAD_COMP_MED_ASSOC_PROF_CURR_ID                : 'Trad comp med assoc prof',
        HW_HEALTH_CARE_ASSISTS_CURR_ID                     : 'Health care assists',
        HW_HOME_BASED_PERS_CARE_WORKERS_CURR_ID            : 'Home based pers care workers',
        HW_PERS_CARE_WORKERS_HEALTH_SERV_NOT_CLASS_CURR_ID : 'Pers care workers health serv not class',
        HW_PHYSIOTHERAPISTS_CURR_ID                        : 'Physiotherapists',
        HW_PHYSIOTHERAPY_TECHS_ASSISTS_CURR_ID             : 'Physiotherapy techs assists',
        HW_HEALTH_ASSOC_PROF_NOT_CLASS_CURR_ID             : 'Health assoc prof not class',
        HW_PSYCHOLOGISTS_CURR_ID                           : 'Psychologists',
        HW_SOCIAL_WORK_COUNSELLING_PROF_CURR_ID            : 'Social work counselling prof',
        HW_SOCIAL_WORK_ASSOC_PROF_CURR_ID                  : 'Social work assoc prof',
}

HW_modvar_shapes = {
    

        hwmvt.HW_VERSION_TAG_V1                                : gb_int_shape('HW Version'),
        hwmvt.HW_DATA_CHANGED_TAG_V1                           : gb_bool_shape('HW Data Changed'), 
        hwmvt.HW_PROJ_VALID_TAG_V1                             : gb_bool_shape('HW Project Valid'), 
        hwmvt.HW_FILE_NAME_TAG_V1                              : gb_str_shape('HW File Name'),                          
        hwmvt.HW_PROJ_VALID_DATE_TAG_V1                        : gb_str_shape('HW Project Valid Date'),    
        hwmvt.HW_COST_INSERV_TRAIN_BY_PERC_SALARY_TAG_V1       : gb_bool_shape('HW Cost Inserv Train By Perc Salary'),
        hwmvt.HW_SHOW_PRESERV_TRAIN_BY_INSTITUTION_TAG_V1      : gb_bool_shape('HW Show Preserv Train By Institution'),  
        hwmvt.HW_STAFF_TYPE_RECORDS_TAG_V1                     : gb_shape('HW Staff Type Records', '1D dict',
                                                                          dim=[hw_staff_dim],
                                                                          indicies=[hw_staff_id],
                                                                          innerShape={
                                                                              'staffTypeID':gb_str_shape('Staff Type ID'), 
                                                                              'stringConstant':gb_str_shape('String Constant'), 
                                                                              'name':gb_str_shape('Name'), 
                                                                              'active': gb_bool_shape('Active'), 
                                                                              'custom': gb_bool_shape('Custom'),
                                                                              'staffDivisionMstID': gb_int_shape('Staff Division Mst ID'),
                                                                              'defStaffTypeMapMstID': gb_int_shape('Default Staff Type Map Mst ID'),
                                                                              'numInBaseYear': gb_shape('Number in Base Year', '1D list'), 
                                                                              'annualSalary': gb_float_shape('Annual Salary'), 
                                                                              'annualRealSalaryIncrease':gb_float_shape('Annual Real Salary Increase'), 
                                                                              'annualBenefits':gb_float_shape('Annual Benefits'), 
                                                                              'annualBenefitsAdjustment': gb_float_shape('Annual Benefits Adjustment'), 
                                                                              'percTimeSpentNotTrainServ': gb_float_shape('Percentage Time Spent Not Train Serv'), 
                                                                              'avgDaysWorkedPerYear':gb_float_shape('Average Days Worked Per Year'), 
                                                                              'hoursInWorkDay':gb_float_shape('Hours in Work Day'), 
                                                                              'currAttritPerYear':gb_shape('Current Attrition Per Year', '1D float',
                                                                                                           dim=[hw_attrition_type_dim],
                                                                                                           indicies=[hw_attrition_type_ids]),
                                                                              'percReceivAttritReducIncents': gb_shape('Percentage Receive Attrition Reduction Incentives', '1D float',
                                                                                                                       dim=[hw_retention_type_dim],
                                                                                                                       indicies=[hw_retention_type_ids]),
                                                                              'annCostAttrReducIncents':gb_shape('Annual Cost Attrition Reduction Incentives',  '1D float',
                                                                                                                       dim=[hw_retention_type_dim],
                                                                                                                       indicies=[hw_retention_type_ids]),
                                                                              'preservNumSchools':gb_int_shape('Preserv Number Schools'), 
                                                                              'preservYearsTraining':gb_int_shape('Preserv Years Training'), 
                                                                              'preservAnnCostPerStudent': gb_float_shape('Preserv Annual Cost Per Student'), 
                                                                              'preservCurrEnrollment':gb_int_shape('Preserv Current Enrollment'), 
                                                                              'gradsMostRecentYear':gb_int_shape('Graduates Most Recent Year'), 
                                                                              'trainingInstNames':gb_shape('Training Institution Names', '1D str',     
                                                                                                            dim=[hw_institutions_dim],
                                                                                                            indicies=[hw_institution_id]),
                                                                              'avgYearsTrainingByInst':gb_shape('average years training by institution', '1D float',     
                                                                                                            dim=[hw_institutions_dim],
                                                                                                            indicies=[hw_institution_id]), 
                                                                              'annualAvgCostPerStudentByInst':gb_shape('annual average cost per student by institution', '1D float',     
                                                                                                            dim=[hw_institutions_dim],
                                                                                                            indicies=[hw_institution_id]), 
                                                                              'currIntakeFirstYearStudentsByInst':gb_shape('current intake first year students by institution', '1D float',     
                                                                                                            dim=[hw_institutions_dim],
                                                                                                            indicies=[hw_institution_id]), 
                                                                              'gradsMostRecentYearByInst':gb_shape('graduates most recent year by institution', '1D float',     
                                                                                                            dim=[hw_institutions_dim],
                                                                                                            indicies=[hw_institution_id]), 
                                                                              'totalAnnualRecruitment':gb_int_shape('Total Annual Recruitment'), 
                                                                              'percNewGradsPublicSector':gb_float_shape('Percentage New Graduates Public Sector'), 
                                                                              'inservTrainCostAsPercSalary':gb_float_shape('Inserv Training Cost as Percentage Salary'), 
                                                                              'targetSettingMethod':gb_shape('Target Setting Method', '1D int',
                                                                                                             dim=[hw_staffing_levels_dim],
                                                                                                             indicies=[hw_staffing_levels_id]),
                                                                              'targetStaffPopRatio': gb_shape('Target Staff Population Ratio', '1D float',
                                                                                                             dim=[hw_staffing_levels_dim],      
                                                                                                             indicies=[hw_staffing_levels_id]),
                                                                              'targetNumPerFacility':gb_shape('Target Number Per Facility', '2D float',
                                                                                                              dim=[hw_staffing_levels_dim, hw_facility_types_dim],
                                                                                                              indicies=[hw_staffing_levels_id, hw_facility_types_id]),
                                                                              'targetScaleUpExistingPlans':gb_shape('Target Scale Up Existing Plans', '2D float',
                                                                                                                    dim=[hw_staffing_levels_dim, gb_year_dim],
                                                                                                                    indicies=[hw_staffing_levels_id, gb_year_indices]),
                                                                              'targetEnrollment': gb_year_shape('Target Enrollment'),    
                                                                              'targetGraduation':gb_year_shape('Target Graduation'),
                                                                              'targetRetentIncentLevels': gb_shape('Target retention incent levels',  '2D float',
                                                                                                                       dim=[hw_retention_type_dim, gb_year_dim],
                                                                                                                       indicies=[hw_retention_type_ids, gb_year_indices],
                                                                                                                       contingent=True),
                                                                          },
                                                                          has_default=False
                                                                          
                                                                 ),        
        hwmvt.HW_INSERV_TRAIN_TYPE_RECORDS_TAG_V1              : gb_shape('HW Staff Type Records', '1D dict',
                                                                          dim=[hw_staff_trained_dim],
                                                                          indicies=[hw_staff_trained_id],
                                                                          innerShape={  
                                                                                'staffTypeToBeTrainedID': gb_str_shape('staff type to be trained ID'),
                                                                                'name': gb_str_shape('Name'),
                                                                                'costPerStaffTrained': gb_float_shape('Cost Per Staff Trained'),
                                                                                'numStaffToBeTrained': gb_year_shape('Number of Staff to be Trained')
                                                                          }
                                                                        ),
        
        hwmvt.HW_ATTRITION_TYPE_RECORDS_TAG_V1                 : gb_shape('Current Attrition Per Year', '1D dict',
                                                                           dim=[hw_attrition_type_dim],
                                                                           indicies=[hw_attrition_type_ids],
                                                                           innerShape={    
                                                                                 'attrTypeID': gb_str_shape('Attrition Type ID'),
                                                                                 'name': gb_str_shape('Name')
                                                                        }
                                                                ),
        hwmvt.HW_RETENTION_INCENTIVE_RECORDS_TAG_V1            :  gb_shape('Annual Cost Attrition Reduction Incentives',  '1D dict',
                                                                           dim=[hw_retention_type_dim],
                                                                           indicies=[hw_retention_type_ids],
                                                                           innerShape={
                                                                               'retentIncentID': gb_str_shape('Retention Incentive ID'),
                                                                               'name': gb_str_shape('Name'),
                                                                               'percReductionAttrition': gb_shape('Percentage Reduction Attrition', '1D float',
                                                                                                                        dim=[hw_attrition_type_dim],
                                                                                                                        indicies=[hw_attrition_type_ids])
                                                                               
                                                                           }, 
                                                                           has_default=False
                                                                ),
        hwmvt.HW_TARG_SCALE_UP_VISITED_TAG_V1                  : gb_shape('Target scale up visited', '1D bool',
                                                                          dim=[hw_staffing_levels_dim],
                                                                          indicies=[hw_staffing_levels_id]),
        hwmvt.HW_TARG_ENROLLMENT_VISITED_TAG_V1                : gb_bool_shape('HW Target Enrollment Visited'),                
        hwmvt.HW_TARG_GRADUATION_VISITED_TAG_V1                : gb_bool_shape('HW Target Graduation Visited'),
        hwmvt.HW_TARG_RETENT_INCENT_LEVELS_VISITED_TAG_V1      : gb_bool_shape('HW Target Retention Incentive Levels Visited'),
        hwmvt.HW_PROG_MGMT_DATA_TAG_V1                         : gb_shape('HW Program Management Data',  'dict',
                                                                           innerShape={
                                                                              'mainRowCosts': gb_shape('Main row costs', '2D float',
                                                                                dim=[hw_pc_standard_rows_dim, gb_year_dim],
                                                                                indicies=[hw_pc_standard_rows_id, gb_year_indices])
                                                                        },
                                                                           has_default=False
                                                                ),

        # # Outputs / results 
        hwmvt.HW_RES_TOTAL_STAFF_TAG_V1                         : gb_shape('Result total staff',  '3D float',
                                                                            dim=[hw_staffing_levels_dim, hw_staff_types_dim, gb_year_dim],
                                                                            indicies=[hw_staffing_levels_id, hw_staff_types_id, gb_year_indices]),
                                                                                
        hwmvt.HW_RES_TOTAL_COMPENSATION_TAG_V1                  : gb_shape('Result total compensation',  '4D float',
                                                                            dim=[hw_staffing_levels_dim, hw_staff_types_dim, hw_staff_comp_dim, gb_year_dim],
                                                                            indicies=[hw_staffing_levels_id, hw_staff_types_id, hw_staff_comp_id, gb_year_indices]),
        hwmvt.HW_RES_STAFF_NEEDING_TO_BE_HIRED_TAG_V1           : gb_shape('Result staff needing to be hired',  '3D float',
                                                                            dim=[hw_staffing_levels_dim, hw_staff_types_dim, gb_year_dim],
                                                                            indicies=[hw_staffing_levels_id, hw_staff_types_id, gb_year_indices]),

        hwmvt.HW_RES_ATTRITION_RATE_TAG_V1                      : gb_shape('Result attrition rate',  '2D float',
                                                                            dim=[hw_staff_types_dim, gb_year_dim],
                                                                            indicies=[hw_staff_types_id, gb_year_indices]),
        hwmvt.HW_RES_INSERV_TRAIN_COSTS_BY_STAFF_TAG_V1         : gb_shape('Result inservice training costs by staff',  '3D float',
                                                                            dim=[hw_staffing_levels_dim, hw_staff_types_dim, gb_year_dim],
                                                                            indicies=[hw_staffing_levels_id, hw_staff_types_id, gb_year_indices]),

        hwmvt.HW_RES_INSERV_TRAIN_COSTS_BY_TRAIN_TYPE_TAG_V1    : gb_shape('Result preserv training costs',  '2D float',
                                                                            dim=[hw_inservice_dim, gb_year_dim],
                                                                            indicies=[hw_inservice_id, gb_year_indices]),
        hwmvt.HW_RES_PRESERV_TRAIN_COSTS_TAG_V1                 : gb_shape('Result preserv training costs',  '2D float',
                                                                            dim=[hw_staff_types_dim, gb_year_dim],
                                                                            indicies=[hw_staff_types_id, gb_year_indices]),
        hwmvt.HW_RES_STAFF_PER_10K_POP_TAG_V1                   : gb_shape('Result staff per 10k population',  '3D float',
                                                                            dim=[hw_staffing_levels_dim, hw_staff_types_dim, gb_year_dim],
                                                                            indicies=[hw_staffing_levels_id, hw_staff_types_id, gb_year_indices]),

        hwmvt.HW_STATIC_STAFF_LIST_META_TAG_V1                  : gb_shape('HW Staff Type Records', '1D dict',
                                                                          dim=[hw_static_staff_dim],
                                                                          indicies=[hw_static_staff_id],
                                                                          innerShape={
                                                                              'mstID' : gb_str_shape('MST ID'),
                                                                              'name' : gb_str_shape('Name'),
                                                                              'stringConstant' : gb_str_shape('String Constant'),
                                                                          }
        )

}


from AvenirCommon.Util import GBRange
from SpectrumCommon.Const.DP import *

dp_default_data_dim = 'dp_default_data_dim'
dp_default_data_indices = {
    'Default' : DP_Default,
    'Data' : DP_Data,   
}

dp_regions_dim = 'dp_regions_dim'
dp_regions_indices = {
    region : DP_GlobalRegions[region] for region in DP_GlobalRegions
}

# ARV Regimen Period
dp_arv_regimen_period_dim = 'dp_arv_regimen_period'
dp_arv_regimen_period_indices = {
    'PrenatalProphylaxis'   : DP_PrenatalProphylaxis,
    'PostnatalProphylaxis'  : DP_PostnatalProphylaxis,
    'AnnDropPostnatalProph' : DP_AnnDropPostnatalProph
}
# ARV Regimen Treatments
dp_arv_regimen_treatments_dim = 'dp_arv_regimen_treatments'
dp_arv_regimen_treatments_indices = {
    'NoProphylaxis'         : DP_NoProphylaxis,
    'SingleDoseNevir'       : DP_SingleDoseNevir,
    'DualARV'               : DP_DualARV,
    'OptA'                  : DP_OptA,
    'OptB'                  : DP_OptB,
    'TripleARTBefPreg'      : DP_TripleARTBefPreg,
    'TripleARTDurPreg'      : DP_TripleARTDurPreg,
    'TripleARTDurPreg_Late' : DP_TripleARTDurPreg_Late,
    'TotalTreat'            : DP_TotalTreat,
}

dp_percent_or_number_dim = 'dp_percent_or_number'
dp_percent_or_number_indices = {
    'Number'    : DP_Number,
    'Percent'   : DP_Percent,
    'NumAndPer' : DP_NumAndPer
}

dp_upd_source_indices = {
'Pop' : DP_UPD_Pop,            
'TFR' : DP_UPD_TFR,            
'LE' : DP_UPD_LE,             
'AgeMortality' : DP_UPD_AgeMortality,   
'ChildMortality' : DP_UPD_ChildMortality, 
'Migration' : DP_UPD_Migration      
}

dp_surv_rate_indices = {
    (str(a)) : a for a in GBRange(0, DP_MAX_SURVRATE)
}

dp_asfr_table_indices = {
    'UNAfrica'  : 1,
    'UNArab'    : 2,
    'UNAsia'    : 3,
    'Average'   : 4
}

dp_asfr_col_indices = {
    ('Col ' + str(col)) : col for col in GBRange(0, DP_ASFR_NumCols)
}
dp_asfr_row_indices = {
    ('Row ' + str(row)) : row for row in GBRange(0, DP_ASFR_NumRows)
}

dp_infant_feed_months_dim = 'infant_feed_months'
dp_infant_feed_months_indices = {
    ('Month ' + str(m)) : m for m in GBRange(0, DP_InfantFeedingMths)
}

dp_infant_feed_types_dim = 'infant_feed_types'
dp_infant_feed_types_indices = {
  'No breastfeeding'        : DP_NoBF,
  'Exclusive breastfeeding' : DP_ExBF
}

dp_PMTCT_dim = 'PMTCT'
dp_PMTCT_indices = {
  'Not in PMTCT' : DP_NotInPMTCT,
  'In PMTCT'     : DP_InPMTCT
}

dp_elig_treat_children_dim = 'elig_treat_children'
dp_elig_treat_children_indices = {
  'Age < 11 months'     : DP_AgeLT11Mths,
  'Age 12 to 35 months' : DP_Age12to35Mths,
  'Age 35 to 59 months' : DP_Age35to59Mths,
  'Age > 5 years'       : DP_AgeGT5Years,
}

dp_ART_delivery_dim = 'ART_delivery'
dp_ART_delivery_indices = {
    'On ART at delivery'       : DP_OnARTAtDelivery,
    'Starting ART at delivery' : DP_StartingARTAtDelivery,
}

dp_ART_coverage_method_dim = 'ART_coverage_method'
dp_ART_coverage_method_indices = {
    'Number or percent'          : DP_NumOrPercent,
    'CD4 percent'                : DP_CD4Percent,  
    'CD4 number'                 : DP_CD4Number,       
    'Number of new ART patients' : DP_NumNewARTPats,
    'Percent by risk group'      : DP_PcntByRiskGrp,
}

dp_CD4_count_cat_dim = 'CD4_count_cat'
dp_CD4_count_cat_indices = {
    'CD4 count > 500'  : DP_CD4_GT500,
    'CD4 count 350-500': DP_CD4_350_500,
    'CD4 count 250-349': DP_CD4_250_349,
    'CD4 count 200-249': DP_CD4_200_249,
    'CD4 count 100-199': DP_CD4_100_199,
    'CD4 count 50-99'  : DP_CD4_50_99,
    'CD4 count < 50'   : DP_CD4_LT50,
}
dp_CD4_count_cat_noLT50_indices = dp_CD4_count_cat_indices.popitem()


dp_GAM_age_group_dim = 'GAM_age_group'
dp_GAM_age_group_indices = {
    'All Ages'      : GB_AllAges,
    'Age 0 to 4'    : GB_A0_4,
    'Age 5 to 9'    : GB_A5_9,
    'Age 10 to 14'  : GB_A10_14,
    'Age 15 to 19'  : GB_A15_19,
    'Age 20 to 24'  : GB_A20_24,
    'Age 25 to 49'  : DP_GAMAG_A25_49,
    'Age 50+'       : DP_GAMAG_A50Plus,
}


dp_pop_elig_treat_regardless_CD4_count_dim = 'pop_elig_treat_regardless_CD4_count'
dp_pop_elig_treat_regardless_CD4_count_indices = {
    'Pregnant women'         : DP_EligTreatPregnantWomen,
    'People with TB and HIV' : DP_EligTreatTB_HIV,
    'Discordant couples'     : DP_EligTreatDiscordantCouples,
    'Sex workers'            : DP_EligTreatSexWorkers,
    'MSM'                    : DP_EligTreatMSM,
    'IDU'                    : DP_EligTreatIDU,
    'Other populations'      : DP_EligTreatOtherPop,
}


dp_per_child_HIV_dim = 'per_child_HIV'
dp_per_child_HIV_indices = {
    'HIV positive children receiving cotrimoxazole' : DP_PerChildHIVPosCot,
    'HIV positive children receiving ART'           : DP_PerChildHIVRecART,
    'HIV positive children age 0-4 receiving ART'   : DP_PerChildHIVRecART0_4,
    'HIV positive children age 5-9 receiving ART'   : DP_PerChildHIVRecART5_9,
    'HIV positive children age 10-14 receiving ART' : DP_PerChildHIVRecART10_14,
}

# ANC Testing Editor constants 
dp_ANC_testing_dim = 'ANC_testing'
dp_ANC_testing_indices = {
    'First ANC visits'                                       : DP_FirstANCVisits,
    'People receiving at least one HIV test'                 : DP_RecAtLeast1HIVTest,
    'People testing positive on the first HIV test'          : DP_PosOnFirstHIVTest,
    'People testing positive for HIV on the first ANC visit' : DP_HIVPosFirstANCVisit,
    'Percent HIV positive on census'                         : DP_PercHIVPosCensus,
    'Number retested'                                        : DP_NumRetested,
    'Number testing positive at retest'                      : DP_NumPosAtRetest,
    'Reported births'                                        : DP_ProgReportedBirths,
    'People testing negative for HIV on the first ANC visit' : DP_HIVNegFirstANCVisit,
}

dp_HIV_status_dim = 'HIV_status'
dp_HIV_status_indices = {
    'Total pop' : DP_D_All,   
    'HIV- pop' : DP_D_HIVNeg,
    'HIV+ pop' : DP_D_HIVPos,
}

dp_num_type_dim = 'Number type'
dp_num_type_indices = {
    'Number' : DP_Number,
    'Percent' : DP_Percent,
    'Lower Bound' : DP_LowerBound,
    'Upper Bound' : DP_UpperBound,
}

dp_shiny90_age_group_dim = 'Shiny90_age_groups'
dp_shiny90_age_group_indices = {
    '15-24' : DP_S90_15_24, 
    '25-34' : DP_S90_25_34, 
    '35-49' : DP_S90_35_49, 
    '15-49' : DP_S90_15_49, 
    '15+' : DP_S90_15Plus,
}

dp_cascade_age_group_dim = 'Cascade_age_groups'
dp_cascade_age_group_indices = {
    'Child' : DP_Cas_AG_Child,      
    'Male' : DP_Cas_AG_AdultMale,  
    'Female' : DP_Cas_AG_AdultFemale,
}

dp_pediatric_age_group_dim = 'Pediatric_age_groups'
dp_pediatric_age_group_indices = {
    '0' : DP_A0,  
    '1-2' : DP_A1t2,
    '3-4' : DP_A3t4,
    '5-14' : DP_A5t14,
}
dp_pediatric_mort_no_ART_LT5_age_group_dim = 'Pediatric_mort_no_ART_LT5_age_groups'
dp_pediatric_mort_no_ART_LT5_age_group_indices = {
    '0' : DP_A0,  
    '1-2' : DP_A1t2,
    '3-4' : DP_A3t4,
}

dp_pediatric_mort_no_ART_GT5_age_group_dim = 'Pediatric_mort_no_ART_GT5_age_groups'
dp_pediatric_mort_no_ART_GT5_age_group_indices = {
    '5-9' : DP_A5t9,
    '10-14' : DP_A10t14,
}

dp_pediatric_CD4_categories_dim = 'Pediatric_CD4_categories'
dp_pediatric_CD4_categories_indices = {
    'A0-4:GT30%, A5-14:Top' : DP_CD4_Per_GT30, #DP_CD4_Ped_Top 
    'A0-4:26-30%, A5-14:GT1000' : DP_CD4_Per_26_30, #DP_CD4_Ped_GT1000
    'A0-4:21-25%, A5-14:750-999' : DP_CD4_Per_21_25, #DP_CD4_Ped_750_999
    'A0-4:16-20%, A5-14:500-749' : DP_CD4_Per_16_20, #DP_CD4_Ped_500_749
    'A0-4:11-15%, A5-14:350-499' : DP_CD4_Per_11_15, #DP_CD4_Ped_350_499
    'A0-4:5-10%, A5-14:200-349' : DP_CD4_Per_5_10, #DP_CD4_Ped_200_349 
    'A0-4:LT5%, A5-14:LT200' : DP_CD4_Per_LT5, #DP_CD4_Ped_LT200  
}
dp_pediatric_CD4_categories_noLT5_indices = dp_pediatric_CD4_categories_indices.popitem()

dp_viral_suppression_dim = 'Viral_suppression_status'
dp_viral_suppression_indices = {
    'NumOnART' : DP_VS_NumOnART,       
    'NumTested' : DP_VS_NumTested,      
    'PLHIVSuppressed' : DP_VS_PLHIVSuppressed,
}

dp_HIV_acquisition_dim = 'ART status or timing of HIV acquisition'
dp_HIV_acquisition_indices = {
    'Children who acquired HIV perinatally' : DP_P_Perinatal,
    'Children who acquired HIV during 0-6 months of breastfeeding' : DP_P_BF0,
    'Children who acquired HIV during 7-12 months of breastfeeding' : DP_P_BF7,
    'Children who acquired HIV during 12+ months of breastfeeding' : DP_P_BF12,
}

dp_mort_rate_dim = 'Mortality rate time periods'
dp_mort_rate_indices = {
    'MortRates_LT12Mo' : DP_MortRates_LT12Mo,
    'MortRates_GT12Mo' : DP_MortRates_GT12Mo,
}

dp_child_treat_effectiveness_dim = 'Effectiveness of Treatment, Child'
dp_child_treat_effectiveness_indices = {
    'Cotrim' : DP_Cotrim,         
    'ChildEffNoART' : DP_ChildEffNoART,  
    'ChildEffWithART' : DP_ChildEffWithART,   
}

dp_child_treat_years_dim = 'Child treatment years'
dp_child_treat_years_indices = {
    ('Treatment year ' + str(i)) : i for i in GBRange(DP_MIN_YEAR, DP_MaxChildTreatmentYears)
}

dp_10y_age_dim = '10y_age'
dp_10y_age_indices = {
    'Age 15 to 24' : DP_CD4_15_24,
    'Age 25 to 34' : DP_CD4_25_34,
    'Age 35 to 44' : DP_CD4_35_44,
    'Age 45 to 54' : DP_CD4_45_54,
}

dp_ART_status_dim = 'ART_status'
dp_ART_status_indices = {
    'NoTreat' : DP_NoTreat,
    'OnART' : DP_OnART,  
}

dp_regimen_transmission_efficacy_MTCT_dim = 'regimen_transmission_efficacy_MTCT'
dp_regimen_transmission_efficacy_MTCT_indices = {
    'No prophylaxis existing inf CD4 < 200'   : DP_NoProphExistInfCD4LT200,
    'No prophylaxis existing inf CD4 200-350' : DP_NoProphExistInfCD4200_350,
    'No prophylaxis existing inf CD4 > 350'   : DP_NoProphExistInfCD4GT350,
    'No prophylaxis incidence inf'            : DP_NoProphIncidentInf,
    'Single dose nev'                         : DP_SingleDoseNev,
    'WHO 2006 dual ARV'                       : DP_WHO2006DualARV,
    'Option A'                                : DP_OptionA,
    'Option B'                                : DP_OptionB,
    'ART starting pre-pregnancy'              : DP_ARTStartPrePreg,
    'ART starting during pregnancy'           : DP_ARTStartDurPreg,
    'ART starting during late pregnancy'      : DP_ARTStartDurPreg_Late, 
}

dp_stage_transmission_efficacy_MTCT_dim = 'stage_transmission_efficacy_MTCT'
dp_stage_transmission_efficacy_MTCT_indices = {
    'Perinatal'            : DP_Perinatal,
    'Breastfeeding < 350'  : DP_BreastfeedingLT350,
    'Breastfeeding >= 350' : DP_BreastfeedingGE350,
}

dp_DALY_weights_dim = 'DALY_weights'
dp_DALY_weights_indices = {
    'DALY CD4 >= 200' : DP_DALY_CD4CountGE200,
    'DALY CD4 < 200'  : DP_DALY_CD4CountLT200,
    'DALY on ART'     : DP_DALY_OnART,
}

dp_adult_child_dim = 'adult_child'
dp_adult_child_indices = {
    'Adult'           : DP_Adult,
    'Child'           : DP_Child,
    'Adult and child' : DP_AdultAndChild,
}

dp_risk_group_dim = 'risk_group'
dp_risk_group_indices = {
    'Injecting drug users'      : DP_InjectingDrugUsers,
    'Men who have sex with men' : DP_MenSexWithMen,
    'Sex workers'               : DP_SexWorkers,
    'Lower risk general pop'    : DP_LowerRiskGenPop,
}

dp_AIDS_deaths_married_dim = 'AIDS_deaths_married'
dp_AIDS_deaths_married_indices = {
    'Percent AIDS deaths': DP_PercAIDSDeaths,
    'Percent married'    : DP_PercMarried,
}

dp_nosocomial_infection_ages_dim = 'nosocomial_infection_ages'
dp_nosocomial_infection_ages_indices = {
    'All ages'     : DP_AllAges,
    'Age 0 to 4'   : DP_A0_4,
    'Age 5 to 9'   : DP_A5_9,
    'Age 10 to 14' : DP_A10_14,
}

dp_CSAVR_reporting_dim = 'CSAVR_reporting'
dp_CSAVR_reporting_indices = {
    'Number reported'    : CSAVRNumReported,
    'Number not reported': CSAVRNumNotReported,
}

dp_CSAVR_source_dim = 'CSAVR_source'   
dp_CSAVR_source_indices = {
    'Source 1': CSAVRSource1,
    'Source 2': CSAVRSource2,
    'Source 3': CSAVRSource3,
} 

dp_CSAVR_ages_dim = 'CSAVR_ages'
dp_CSAVR_ages_indices = {
    'All ages'       : DP_AllAges,
    'Age 0 to 4'     : DP_A0_4,
    'Age 5 to 9'     : DP_A5_9,
    'Age 10 to 14'   : DP_A10_14,
    'Age 15 to 19'   : DP_A15_19,
    'Age 20 to 24'   : DP_A20_24,
    'Age 25 to 29'   : DP_A25_29,
    'Age 30 to 34'   : DP_A30_34,
    'Age 35 to 39'   : DP_A35_39,
    'Age 40 to 44'   : DP_A40_44,
    'Age 45 to 49'   : DP_A45_49,
    'Age 50 to 54'   : DP_A50_54,
    'Age 50 plus'    : DP_A50Plus,
}

dp_CSAVR_CD4_dim = 'CSAVR_CD4'
dp_CSAVR_CD4_indices = {
    'CD4 < 200'      : CSAVR_CD4_LT200,
    'CD4 200 to 350' : CSAVR_CD4_200_350,
    'CD4 350 to 500' : CSAVR_CD4_350_500,
    'CD4 500 plus'   : CSAVR_CD4_500Plus,
}

dp_CSAVR_CD4_2_dim = 'CSAVR_CD4_2'
dp_CSAVR_CD4_2_indices = {
    'CD4 < 200'      : CSAVR_CD4_LT200,
    'CD4 200 to 350' : CSAVR_CD4_200_350,
    'CD4 350 to 500' : CSAVR_CD4_350_500,
    'CD4 500 plus'   : CSAVR_CD4_500Plus,
    'CD4 all'        : CSAVR_CD4_All,
}
    
dp_AIDS_deaths_all_ages_num_dim = 'AIDS_deaths_all_ages_num'
dp_AIDS_deaths_all_ages_num_indices = {
    'Median'      : DP_Median,
    'Under count' : DP_UnderCount,
}

dp_fitting_incidence_option_dim = 'fitting_incidence_option'
dp_fitting_incidence_option_indices = {
    'PLHIV'                : DP_PLHIV,
    'New HIV cases'        : DP_NewHIVCases,
    'AIDS deaths'          : DP_AIDSDeaths,
    'HIV positive deaths'  : DP_DeathsHIVPos,
    'CD4 dist at diagnosis': DP_CD4DistAtDiag,
}

dp_CSAVR_fit_types_dim = 'CSAVR_fit_types'
dp_CSAVR_fit_types_indices = {
    'Fit not calculated' : DP_FitNotCalculated,
    'Double logistic'    : DP_DoubleLogistic,
    'Single logistic'    : DP_SingleLogistic,
    'Spline 5 knots'     : DP_Spline5Knots,
    'Logistic'           : DP_rLogistic,
    'Spline 4 knots'     : DP_Spline4Knots,
    'Spline 3 knots'     : DP_Spline3Knots,
}

dp_CSAVR_uncertainty_types_dim = 'CSAVR_uncertainty_types'
dp_CSAVR_uncertainty_types_indices = {
    'Uncertainty burn-in'           : CSAVR_Unc_BurnIn,
    'Uncertainty number of samples' : CSAVR_Unc_NumSamples,
}

dp_CSAVR_adjust_IRRs_dim = 'CSAVR_adjust_IRRs'
dp_CSAVR_adjust_IRRs_indices = {
    'Sex ratio'           : DP_CSAVR_SexRatio,
    'Distribution of HIV' : DP_CSAVR_DistOfHIV,
}

dp_age_ratio_patterns_dim = 'age_ratio_patterns'
dp_age_ratio_patterns_indices = {
    'Generalized'    : DP_Generalized_Index,
    'Concentrated'   : DP_NonIDUConcentrated_Index,
    'Concentrated 2' : DP_IDUConcentrated_Index,
}

dp_prev_surveys_dim = 'prev_surveys'
dp_prev_surveys_indices = {
    'Prev survey 1' : DP_PrevSurvey1,
    'Prev survey 2' : DP_PrevSurvey2,
    'Prev survey 3' : DP_PrevSurvey3,
    'Prev survey 4' : DP_PrevSurvey4,
    'Prev survey 5' : DP_PrevSurvey5,
    'Prev survey 6' : DP_PrevSurvey6,
    'Prev survey 7' : DP_PrevSurvey7,
    'Prev survey 8' : DP_PrevSurvey8,
}

dp_sample_sizes_dim = 'sample_sizes'
dp_sample_sizes_indices = {
    'Number'               : DP_Number,
    'Percent'              : DP_Percent,
    'Lower bound'          : DP_LowerBound,
    'Upper bound'          : DP_UpperBound,
    'Weighted sample size' : DP_WeightedSampleSize,
}

dp_sample_sizes_2_dim = 'sample_sizes_2'
dp_sample_sizes_2_indices = {
    'Number'               : DP_Number,
    'Percent'              : DP_Percent,
    'Lower bound'          : DP_LowerBound,
    'Upper bound'          : DP_UpperBound,
}

# Key Populations
dp_key_pops_1_dim = 'key_pops_1'
dp_key_pops_1_indices = {
    'FSW'  : DP_KP_FSW,
    'MSM'  : DP_KP_MSM,
    'PWID' : DP_KP_PWID,
    'TG'   : DP_KP_TG,
}

dp_key_pops_2_dim = 'key_pops_2'
dp_key_pops_2_indices = {
    'PSE'            : DP_KP_PSE,
    'HIV prevention' : DP_KP_HIVPrev,
    'ART coverage'   : DP_KP_ARTCov,
    'New infections' : DP_KP_NewInfs,
}

dp_ART_cov_validation_ages_dim = 'ART_coverage_validation'
dp_ART_cov_validation_ages_indices = {
    'Validation ages 0 to 14'  : DP_Val_A0_14,
    'Validation ages 15 to 49' : DP_Val_A15_49,
    'Validation ages 50 plus'  : DP_Val_A50Plus,
}

dp_incidence_option_dim = 'incidence_option'
dp_incidence_option_indices = {
    'Direct incidence input' : DP_DirectIncidenceInputOpt,
    'EPP incidence'          : DP_EPPIncOpt,
    'AEM incidence'          : DP_AEMIncOpt,
    'Fit to program data'    : DP_FitToProgramDataOpt,
    'Fit to mortality data'  : DP_FitToMortalityDataOpt,
    'ECDC'                   : DP_ECDCOpt,
}

dp_EPP_epi_1_dim = 'EPP_epi_1'
dp_EPP_epi_1_indices = {
    'Country ID'   : DP_CountryID,
    'Min epidemic' : DP_MinEpidemic,
    'Max epidemic' : DP_MaxEpidemic,
}

dp_EPP_epi_2_dim = 'EPP_epi_2'
dp_EPP_epi_2_indices = {
    'Min epidemic' : DP_MinEpidemic,
    'Max epidemic' : DP_MaxEpidemic,
}
 
dp_CD4_count_cats_children_5_14_dim = 'dp_CD4_count_cats_children_5_14'
dp_CD4_count_cats_children_5_14_indices = {
    'CD4 top'        : DP_CD4_Ped_Top,
    'CD4 > 1000'     : DP_CD4_Ped_GT1000,
    'CD4 750 to 999' : DP_CD4_Ped_750_999,
    'CD4 500 to 749' : DP_CD4_Ped_500_749,
    'CD4 350 to 499' : DP_CD4_Ped_350_499,
    'CD4 200 to 349' : DP_CD4_Ped_200_349,
    'CD4 < 200'      : DP_CD4_Ped_LT200,
}

dp_HIV_mort_with_ART_treat_interval_dim = 'dp_HIV_mort_with_ART_treat_interval'
dp_HIV_mort_with_ART_treat_interval_indices = {
    '0 to 6 months'  : DP_0to6monthsTreat,
    '7 to 12 months' : DP_7to12monthsTreat,
    '> 12 months'    : DP_GT12monthsTreat,
}

dp_HIV_status_dim = 'dp_HIV_status'
dp_HIV_status_indices = {
    'All'                         : DP_H_All,
    'No'                          : DP_H_No,
    'Asymptomatic'                : DP_H_Asym,
    'Need treatment'              : DP_H_NeedTx,
    'On first-line ART'           : DP_H_OnFLArt,
    'Needing second-line ART'     : DP_H_NeedSL,
    'On second-line ART'          : DP_H_OnSLArt,
    'Breastfeeding < 6 months'    : DP_H_BF_LT6_Asym,
    'Breastfeeding 6 to 12 months': DP_H_BF_6to12_Asym,
}

dp_CD4_dist_not_on_ART_dim = 'CD4_dist_not_on_ART'
dp_CD4_dist_not_on_ART_indices = {
    '0 to 4'  : DP_CD4_0t4,
    '5 to 14' : DP_CD4_5t14,    
}

dp_HIV_duration_status_timing_dim = 'HIV_duration_status_timing'
dp_HIV_duration_status_timing_indices = {
    'All'                : DP_D_All,
    'HIV negative'       : DP_D_HIVNeg,
    'HIV positive'       : DP_D_HIVPos,
    'ART 1 to 6 months'  : DP_D_ARTlt6m,
    'ART 6 to 12 months' : DP_D_ART6to12m,
    'ART > 12 months'    : DP_D_ARTgt12m,
}

dp_subpop_summary_ind_dim = 'subpop_summary_ind'
dp_subpop_summary_ind_indices = {
    'Adults - Total HIV population' : DP_HIVPopTot_Ad,
    'Adults - Male HIV population' : DP_HIVPopMale_Ad,
    'Adults - Female HIV population' : DP_HIVPopFemale_Ad,
    'Adults - Adult prevalence' : DP_AdultPrev_Ad,
    'Adults - Total new HIV infections' : DP_NewHIVInfectTot_Ad,
    'Adults - Male new HIV infections' : DP_NewHIVInfectMale_Ad,
    'Adults - Female new HIV infections' : DP_NewHIVInfectFemale_Ad,
    'Adults - Adult HIV incidence' : DP_AdultHIVInc_Ad,
    'Adults - Total annual AIDS deaths' : DP_AnnAIDSDeathTot_Ad,
    'Adults - Male annual AIDS deaths' : DP_AnnAIDSDeathMale_Ad,
    'Adults - Female annual AIDS deaths' : DP_AnnAIDSDeathFemale_Ad,
    'Adults - Total number on ART' : DP_NumOnARTTot_Ad,
    'Adults - Male number on ART' : DP_NumOnARTMale_Ad,
    'Adults - Female number on ART' : DP_NumOnARTFemale_Ad,
    'Adults - Total number needing ART' : DP_NumNeedARTTot_Ad,
    'Adults - Male number needing ART' : DP_NumNeedARTMale_Ad,
    'Adults - Female number needing ART' : DP_NumNeedARTFemale_Ad,
    'Adults - Total population 15 plus' : DP_AdPop15PlusTot_Ad,
    'Adults - Male population 15 plus' : DP_AdPop15PlusMale_Ad,
    'Adults - Female population 15 plus' : DP_AdPop15PlusFemale_Ad,
    'Children - Total HIV population' : DP_HIVPopTot_Child,
    'Children - Male HIV population' : DP_HIVPopMale_Child,
    'Children - Female HIV population' : DP_HIVPopFemale_Child,
    'Children - Total new HIV infections' : DP_NewHIVInfectTot_Child,
    'Children - Male new HIV infections' : DP_NewHIVInfectMale_Child,
    'Children - Female new HIV infections' : DP_NewHIVInfectFemale_Child,
    'Children - Total annual AIDS deaths' : DP_AnnAIDSDeathTot_Child,
    'Children - Male annual AIDS deaths' : DP_AnnAIDSDeathMale_Child,
    'Children - Female annual AIDS deaths' : DP_AnnAIDSDeathFemale_Child,
    'Children - Number needing PMTCT' : DP_NeedPMTCT_Child,
    'Children - Number on PMTCT' : DP_OnPMTCT_Child,
    'Children - Total population 0 to 14' : DP_AdPop0_14Tot_Child,
    'Children - Male population 0 to 14' : DP_AdPop0_14Male_Child,
    'Children - Female population 0 to 14' : DP_AdPop0_14Female_Child,
}

dp_orphans_dim = 'orphans'
dp_orphans_indices = {
    'Total'             : DP_Total_orphans,
    'Maternal AIDS'     : DP_Maternal_AIDS,
    'Maternal non-AIDS' : DP_Maternal_Non_AIDS,
    'Paternal AIDS'     : DP_Paternal_AIDS,
    'Paternal non-AIDS' : DP_Paternal_Non_AIDS,
    'Dual AIDS'         : DP_Dual_AIDS,
    'Dual non-AIDS'     : DP_Dual_Non_AIDS,
    'Total AIDS'        : DP_Total_AIDS,
    'Total non-AIDS'    : DP_Total_Non_AIDS,
    'Dual total'        : DP_Dual_Total,
    'Maternal'          : DP_Maternal_Orphans,
    'Paternal'          : DP_Paternal_Orphans,
}

dp_ALB_dim = ''
dp_ALB_indices = {
    'Maternal AIDS'     : DP_MaternalAIDS_ALB,
    'Maternal non-AIDS' : DP_MaternalNonAIDS_ALB,
    'Paternal AIDS'     : DP_PaternalAIDS_ALB,
    'Paternal non-AIDS' : DP_PaternalNonAIDS_ALB,
    'Dual AIDS'         : DP_DualAIDS_ALB,
    'Dual non-AIDS'     : DP_DualNonAIDS_ALB,
    'Dual'              : DP_Dual_ALB,
}

dp_orphans_total_dim = 'orphans_total'
dp_orphans_total_indices = {
    'Maternal total' : DP_MaternalTotal,
    'Paternal total' : DP_PaternalTotal,
    'Dual total'     : DP_DualTotal,
    'Total'          : DP_TotalOrphans,
}

dp_orphans_total_2_dim = 'orphans_total_2'
dp_orphans_total_2_indices = {
    'Maternal total'       : DP_MaternalTotal,
    'Paternal total'       : DP_PaternalTotal,
    'Dual total'           : DP_DualTotal,
    'Total'                : DP_TotalOrphans,
    'All non-AIDS orphans' : DP_AllNonAIDSOrphans,
    'All AIDS orphans'     : DP_AllAIDSOrphans,
}

dp_AIDS_dim = 'AIDS'
dp_AIDS_indices = {
    'Total'    : DP_Total_AIDS_NonAIDS,
    'AIDS'     : DP_AIDS,
    'Non-AIDS' : DP_Non_AIDS,
}

dp_time_steps_dim = 'time_steps'
dp_time_steps_indices = {
    ('Time step ' + str(t)) : t for t in GBRange(0, 10)
}

dp_regions_dim = 'regions'
dp_regions_indices = {
    'Asia'                            : DP_Asia,
    'Central Africa'                  : DP_CAfrica,
    'Developed countries'             : DP_DevCountries,
    'Eastern Africa'                  : DP_EAfrica,
    'Eastern Europe'                  : DP_EEurope,
    'Latin American and Caribbean'    : DP_LatAmerCarr,
    'Northern Africa and Middle East' : DP_NAfrMiddEast,
    'Southern Africa'                 : DP_SAfrica,
    'Western Africa'                  : DP_WAfrica,
}

dp_15_19_coefficients_dim = '15_19_coefficients'
dp_15_19_coefficients_indices = {
    'Intercept' : DP_Intercept,
    'Slope'     : DP_Slope,
}

dp_HIV_ART_age_dim = 'HIV_ART_age'
dp_HIV_ART_age_indices = {
    '1 to 6 months' : DP_HIVARTlt6m,
    '> 6 months'    : DP_HIVARTgt6m,
    '> 12 months'   : DP_HIVARTgt12m,
}

dp_ART_regimen_PMTCT_costing_dim = 'ART_regimen_PMTCT_costing'
dp_ART_regimen_PMTCT_costing_indices = {
    'Option A breastfeeding' : DP_OptB_BF,
    'Option B breastfeeding' : DP_OptB_BF,
}

dp_advanced_opt_new_ART_patient_alloc_method_dim = 'advanced_opt_new_ART_patient_alloc_method'
dp_advanced_opt_new_ART_patient_alloc_method_indices = {
    'Exp mortality'       : DP_AdvOpt_ART_ExpMort,
    'Proportion eligible' : DP_AdvOpt_ART_PropElig,
}

dp_PMTCT_2_dim = 'PMTCT_2'
dp_PMTCT_2_indices = {
    'Number of women'      : PMTCT_NumberOfWomen,
    'New child infections' : PMTCT_NewChildInfections,
    'Transmission rate'    : PMTCT_TransmissionRate,
}

dp_MTCT_dim = 'MTCT'
dp_MTCT_indices = {
    'ARV before perinatal'             : MTCT_Ind_ARVBeforePerinatal,
    'ARV during perinatal'             : MTCT_Ind_ARVDuringPerinatal,
    'ARV during dropout child inf'     : MTCT_Ind_ARVDuringDropoutChildInf,
    'ARV late perinatal'               : MTCT_Ind_ARVLatePerinatal,
    'No ARVperinatal'                  : MTCT_Ind_NoARVPerinatal,
    'Incident perinatal'               : MTCT_Ind_IncidentPerinatal,
    'ARV before breastfeeding'         : MTCT_Ind_ARVBeforeBF,
    'ARV during breastfeeding'         : MTCT_Ind_ARVDuringBF,
    'Dropped ART during breastfeeding' : MTCT_Ind_DroppedARTDurBF,
    'ARV late breastfeeding'           : MTCT_Ind_ARVLateBF,
    'No ART breastfeeding'             : MTCT_Ind_NoARVBF,
    'Incident breastfeeding'           : MTCT_Ind_IncidentBF,
}

dp_child_weight_bands_kg_dim = 'child_weight_bands_kg'
dp_child_weight_bands_kg_indices = {
    '3 to 5.9'   : DP_Kgs_3_5pt9,
    '6 to 9.9'   : DP_Kgs_6_9pt9,
    '10 to 13.9' : DP_Kgs_10_13pt9,
    '14 to 19.9' : DP_Kgs_14_19pt9,
    '20 to 24.9' : DP_Kgs_20_24pt9,
    '25 to 34.9' : DP_Kgs_25_34pt9,
    '35 plus'    : DP_Kgs_35_Plus,
}

DP_Cas_AG_dim = 'cas_age_group'
DP_Cas_AG_indices = {
    DP_Cas_AG_Child           : 'Children',
    DP_Cas_AG_AdultMale       : 'Adult Males',
    DP_Cas_AG_AdultFemale     : 'Adult Females',
    DP_Cas_AG_AllAges         : 'All ages',
    DP_Cas_AG_AdultBothSexes  : 'Adult Both Sexes'
}


DP_Cas_IND_dim = 'case indicator'
DP_Cas_IND_indices = {
    DP_Cas_IND_Numbers    : 'Numbers',
    DP_Cas_IND_PerElig    : 'Percent eligible',
    DP_Cas_IND_PerPLHIV   : 'Percent PLHIV'
}


DP_Cas_dim = 'case data type'
DP_Cas_indices = {
    DP_Cas_PLHIV          : 'PLHIV',
    DP_Cas_KnowStatus     : 'People who know their HIV status',
    DP_Cas_OnART          : 'People on ART',
    DP_Cas_ViralSuppr     : 'People who are viral suppressed',
}

DP_DetCas_dim = 'Detected case data type'
DP_DetCas_indices = {
    DP_DetCas_KoSDiagOnTreat             : 'Diagnosed on treatment',
    DP_DetCas_KoSDiagNeverTreat          : 'Diagnosed never treated',
    DP_DetCas_KoSDiagPrevOnTreat         : 'Diagnost previously on treatment',
    DP_DetCas_KoSLongStandingInfNotDiag  : 'Long standing infection not diagnosed',
    DP_DetCas_KoSIncidentInfNotDiag      : 'Incident infection not diagnosed',
}


from loguru import logger
from SpectrumCommon.Const.GB import *
from SpectrumCommon.Const.PJ import *
from SpectrumCommon.Const.AM.AMTags import AM_FirstYearOfEpidemicTag
from SpectrumCommon.Modvars.GB.GBCreateProjection import GB_CreateProjection
from SpectrumCommon.Modvars.GB.GBDefs import createProjectionParams
from SpectrumCommon.Modvars.DP.DPUpdateModvars import DP_UpdateModvars
from SpectrumCommon.Modvars.AM.AMUpdateModvars import AM_UpdateModvars
from SpectrumCommon.Modvars.CS.CSUpdateModvars import CS_UpdateModvars, CS_GSKModeTag, CS_PreUpdateModvars
from SpectrumCommon.Modvars.TB.TBUpdateModvars import TB_UpdateModvars, TB_PreUpdateModvars
from SpectrumCommon.Modvars.IC.ICUpdateModvars import IC_UpdateModvars
from SpectrumCommon.Modvars.BG.BGUpdateModvars import BG_UpdateModvars
from SpectrumCommon.Modvars.IS.ISUpdateModvars import IS_UpdateModvars
from SpectrumCommon.Modvars.PC.PCUpdateModvars import PC_UpdateModvars

def GB_UpdateModvars(projection):
    '''
        Updates Modvars (MVs) for a projection that have changed in subsequent Projection Manager releases.

            Parameters:
                projection (dict): All MVs in the projection (before being updated).

            Returns:
                projection (dict): All MVs in the projection (after being updated).

    '''

    module_switch = {
        # GB_GBCode : None,
        # GB_GBCode : None,
        GB_DPCode : DP_UpdateModvars,
        # GB_IVCode : None,
        # GB_FPCode : None,
        GB_AMCode : AM_UpdateModvars,
        # GB_RPCode : None,
        # GB_HVCode : None,
        GB_CSCode : CS_UpdateModvars,
        # GB_IHCode : None,
        # GB_HWCode : None,
        # GB_BACode : None,
        GB_ICCode : IC_UpdateModvars,
        GB_ISCode : IS_UpdateModvars,
        # GB_FSCode : None,
        GB_PCCode : PC_UpdateModvars,
        # GB_LGCode : None,
        GB_TBCode : TB_UpdateModvars,
        GB_BGCode : BG_UpdateModvars,
        # GB_HSCode : None,
        # GB_RNCode : None,
        # GB_GVCode : None,
        # GB_HFCode : None,
        # GB_NCCode : None,
        # GB_TICode : None,
        # GB_MACode : None,
        # GB_GCCode : None,
        # GB_HACode : None,
        # GB_MTCode : None,
        # GB_SICode : None,
        # GB_GTCode : None,
        # GB_PJCode : PJ_UpdateModvars,
    }

    module_pre_updater_switch = {
        GB_TBCode : TB_PreUpdateModvars,
        GB_CSCode : CS_PreUpdateModvars
    }
 
    # Add new modvars and updating
    create_params = createProjectionParams()
    
    create_params.fileName = projection[PJN_TitleTag]
    create_params.country = projection[PJN_ISO3AlphaTag]
    create_params.firstYear = projection[PJN_FirstYearTag]
    create_params.finalYear = projection[PJN_FinalYearTag]
    create_params.modules = projection[PJN_ModulesTag]
    create_params.extra = {
        GB_IHT_ON                 : projection.get(GB_IHTOnTag , False),
        GB_TB_COSTING_ON          : projection.get(GB_TBCostingOnTag , False),
        GB_LIST_COSTING_ON        : projection.get(GB_LCOnTag , False),
        GB_GSKMode                : projection.get(GB_GSKModeTag, projection.get(CS_GSKModeTag, False)),
        # GB_RAPID_SECTORS_ON       : projection.get(GB_RAPIDSectorsOnTag, RP_init_sector_list()),
        GB_TB_ImpactStatistical   : projection.get(GB_TBImpactStatisticalTag , False),
        GB_TB_ImpactDynamical     : projection.get(GB_TBImpactDynamicalTag , False),
        GB_Extra_SubnatCode       : projection.get(PJN_SubNationalCodeTag, 0),
        GB_Extra_subnatName       : projection.get(PJN_SubNationalNameTag, ''),
        GB_Extra_projection_title : projection.get(PJN_TitleTag, ''),
        GB_Extra_Author           : projection.get(PJN_AuthorTag, ''),
        GB_Extra_organization     : projection.get(PJN_OrganizationTag, ''),
        GB_Extra_notes            : projection.get(PJN_NotesTag, ''),
        GB_Extra_appID            : projection.get(PJN_AppIDTag, ''),
        GB_Extra_plantype         : projection.get(PJN_IHTPlanTypeTag, ''),
        GB_Extra_readonly         : projection.get(PJN_ReadOnlyTag, ''),
        
         # Need to use the correct epidemic start year for sex ratio modvar
        GB_Extra_epidemicStartYr  : projection.get(AM_FirstYearOfEpidemicTag, 1980),
    }

    # Contains all the new MVs.
    created_projection = GB_CreateProjection(create_params)
    
    # Make sure all the modules in the projection we just created are in PJN_ModulesTag. If new modules get added for existing
    # projections, they won't be in 'projection[PJN_ModulesTag]' yet.
    projection[PJN_ModulesTag] = []
    for module in created_projection[PJN_ModulesTag]:
        projection[PJN_ModulesTag].append(module)

    if projection[PJN_VersionTag][0] == '0':
        return created_projection
    
    modules = projection[PJN_ModulesTag]
    
    # Pre-update modvars that don't exist in the projection yet
    for md in modules:
        if GB_MOD_STR[md] in module_pre_updater_switch:
            logger.debug(f'PreUpdating {GB_MOD_STR[md]}')
            projection = module_pre_updater_switch[GB_MOD_STR[md]](projection, created_projection)  

    # add any missing modvars from created_projection to projection  
    for tag in created_projection:
        if tag not in projection:
            projection.update({tag: created_projection[tag]})

    # Update all meta modvars
    for tag in projection:
        if '_meta_' in tag.lower():
            if tag in created_projection:
                projection[tag] = created_projection[tag]

    # Update and delete old modvars
    for md in modules:
        if GB_MOD_STR[md] in module_switch:
            logger.debug(f'Updating {GB_MOD_STR[md]}')
            projection = module_switch[GB_MOD_STR[md]](projection, created_projection)  

    # Remove any old tags that are not found in a newly created projection, preserve pseudo-modules that are dynamically added
    projection = {tag:projection[tag] for tag in projection if ((tag in created_projection) or (tag[1:3] in GB_PSEUDO_MOD_ID))} 

    # Once everything is updated, update the version to the latest version
    projection[PJN_VersionTag] = GB_PM_Version
    return projection


import numpy as np

from SpectrumCommon.Const.TB import *


def TB_HighRiskGroupDemographicDefaults(pop_15_plus, c_ep, prev_pct, year_len):
    tb_demographics = np.zeros((TB_HRG_Len, TG_HRG_DemoGraphics_Len, year_len))
    tb_demographics[TB_HRG_Prisoners        ][TB_HRG_Dem_RelRisk] += 004.01
    tb_demographics[TB_HRG_Miners           ][TB_HRG_Dem_RelRisk] += 001.92
    tb_demographics[TB_HRG_SeekingCare      ][TB_HRG_Dem_RelRisk] += 003.0
    tb_demographics[TB_HRG_StructuralRFs    ][TB_HRG_Dem_RelRisk] += 002.5
    tb_demographics[TB_HRG_HighTBPrev       ][TB_HRG_Dem_RelRisk] += 002.5
    tb_demographics[TB_HRG_UntreatedFibrosis][TB_HRG_Dem_RelRisk] += 002.5

    for h in range(0, TB_HRG_Len):
        tb_demographics[h][TG_HRG_Dem_TotalPop] += pop_15_plus  # this is total 15+ pop and should come from DP. We can also create a database entry for it.
        # Proportion of population 15+ in HRG
        tb_demographics[h][TG_HRG_Dem_Prop15p] += 0.001 if h == TB_HRG_HighTBPrev else 0# 0% default for all groups except "General population in settings with ≥ 0.5% general prevalence" which defaults to 3.5%. Al values can be edited by user.
        # Proportion of�ART patients 15 years and older with TB infection (# )
        tb_demographics[h][TG_HRG_Dem_TBInfect] += 0.40# from editor

        # tb_demographics[h][TB_HRG_Dem_RelRisk] += 0.025
        # Proportion of�ART patients 15 years and older with TB disease (# )
        tb_demographics[h][TB_HRG_Dem_TBDisease] = prev_pct*tb_demographics[h][TB_HRG_Dem_RelRisk]

        # Proportion with pulmonary TB, PTB (# )
        tb_demographics[h][TB_HRG_Dem_PTB] += 1-c_ep# from editor

    tb_demographics[TB_HRG_SeekingCare][TB_HRG_Dem_TBDisease] [tb_demographics[TB_HRG_SeekingCare][TB_HRG_Dem_TBDisease] <0.1] = 0.1
    return tb_demographics


# fmt: off
# AIM-specific tags

AM_InitialConditionsTag                     = '<AM_InitialConditions_V1>'
AM_DataSourceTag                            = '<AM_DataSource_V1>'
AM_AMSourcesTag                             = '<AM_AMSources_V1>'
# AM_EdAdSourceTag                            = '<AM_EdAdSource_V1>'
AM_RetroYearsCalibrationTag                 = '<AM_RetroYearsCalibration_V1>'
AM_CD4ThreshHoldTag                         = '<AM_CD4ThreshHold_V1>'
AM_PopsEligTreatTag                         = '<AM_PopsEligTreat_V1>'
AM_AgeHIVChildOnTreatmentTag                = '<AM_AgeHIVChildOnTreatment_V1>'
AM_CD4ThreshHoldAdultsTag                   = '<AM_CD4ThreshHoldAdults_V1>'
AM_InfantFeedingOptionsTag                  = '<AM_InfantFeedingOptions_V1>'
AM_InfantFeedingOptions_MetaTag             = '<AM_InfantFeedingOptions_Meta_V1>'
AM_ARVRegimenTag                            = '<AM_ARVRegimen_V1>'
AM_ARVRegimen_MetaTag                       = '<AM_ARVRegimen_Meta_V1>'
AM_PatientsReallocatedTag                   = '<AM_PatientsReallocated_V1>'
AM_PercentARTDeliveryTag                    = '<AM_PercentARTDelivery_V1>'
AM_PregTermAbortionPerNumTag                = '<AM_PregTermAbortionPerNum_V1>'
AM_PregTermAbortionTag                      = '<AM_PregTermAbortion_V1>'
AM_NumNewARTPatsTag                         = '<AM_NumNewARTPats_V1>'
AM_MedCD4CountInitTag                       = '<AM_MedCD4CountInit_V1>'
# AM_PercLostFollowupTag                      = '<AM_PercLostFollowup_V1>' # Turned into AM_PercInterruptedTag
AM_PercInterruptedTag                       = '<AM_PercInterrupted_V1>'
AM_NumberInitTreatmentReinitsTag            = '<AM_NumberInitTreatmentReinits_V1>'
AM_NumNewlyInitARTTag                       = '<AM_NumNewlyInitART_V1>'
# AM_PercLostFollowupChildTag                 = '<AM_PercLostFollowupChild_V1>' # Turned into AM_PercInterruptedChildTag
AM_PercInterruptedChildTag                  = '<AM_PercInterruptedChild_V1>'
AM_NumberInitTreatmentReinitsChildTag       = '<AM_NumberInitTreatmentReinitsChild_V1>'
AM_NumNewlyInitARTChildTag                  = '<AM_NumNewlyInitARTChild_V1>'
AM_HAARTBySexTag                            = '<AM_HAARTBySex_V1>'
AM_CD4CoverageTag                           = '<AM_CD4Coverage_V1>'
AM_ARTByAgeInputTypeTag                     = '<AM_ARTByAgeInputType_V1>'
AM_ARTByAge5YearAGTag                       = '<AM_ARTByAge5YearAG_V1>'
AM_ARTByAgeGAMAGTag                         = '<AM_ARTByAgeGAMAG_V1>'
AM_CovPopsEligTreatTag                      = '<AM_CovPopsEligTreat_V1>'
AM_ChildTreatInputsTag                      = '<AM_ChildTreatInputs_V1>'
# AM_ChildTreatInputTypeTag                   = '<AM_ChildTreatInputType_V1>'
# AM_CotrimPerNumTag                          = '<AM_CotrimPerNum_V1>'
AM_ChildARTByAgeGroupPerNumTag              = '<AM_ChildARTByAgeGroupPerNum_V1>'
# AM_ChildARTByMonthTag                       = '<AM_ChildARTByMonth_V1>'
# AM_ChildPercentLTFUByMonthTag               = '<AM_ChildPercentLTFUByMonth_V1>'
AM_ANCTestingValuesTag                      = '<AM_ANCTestingValues_V1>'
# AM_NetMigrationPLHIVTag                     = '<AM_NetMigrationPLHIV_V1>'
AM_NewInfectionsLessImmigrantsTag           = '<AM_NewInfectionsLessImmigrants_V1>'
AM_HIVTestingTag                            = '<AM_HIVTesting_V2>'
AM_HIVTestingHTSTestsByAgeTag               = '<AM_HIVTestingHTSTestsByAge_V1>'
AM_Shiny90SurveyDataTag                     = '<AM_Shiny90SurveyData_V1>'
AM_Shiny90SurveyData_MetaTag                = '<AM_Shiny90SurveyData_Meta_V1>'
AM_Shiny90ProgramDataTag                    = '<AM_Shiny90ProgramData_V1>'
AM_Shiny90ProgramData_MetaTag               = '<AM_Shiny90ProgramData_Meta_V1>'
AM_Shiny90AIDSDeathsFYearTag                = '<AM_Shiny90AIDSDeathsFYear_V1>'
AM_Shiny90PopTag                            = '<AM_Shiny90Pop_V1>'
AM_Shiny90NumTestsTag                       = '<AM_Shiny90NumTests_V1>'
AM_Shiny90NumTested12MTag                   = '<AM_Shiny90NumTested12M_V1>'
AM_Shiny90TotalTestsTag                     = '<AM_Shiny90TotalTests_V1>'
AM_Shiny90EverTestedTag                     = '<AM_Shiny90EverTested_V1>'
AM_Shiny90DiagnosedTag                      = '<AM_Shiny90Diagnosed_V1>'
AM_Shiny90NumAwareTag                       = '<AM_Shiny90NumAware_V1>'
AM_Shiny90NumDiagnosesTag                   = '<AM_Shiny90NumDiagnoses_V1>'
AM_Shiny90NumDiagnosesModTag                = '<AM_Shiny90NumDiagnosesMod_V1>'
AM_Shiny90NumLateDiagnosesTag               = '<AM_Shiny90NumLateDiagnoses_V1>'
AM_Shiny90NotDiagHIV1YrTag                  = '<AM_Shiny90NotDiagHIV1Yr_V1>'
AM_Shiny90NumFirstTestHIVNeg_Tag            = '<AM_Shiny90NumFirstTestHIVNeg_V1>'
AM_Shiny90NumReTestHIVNeg_Tag               = '<AM_Shiny90NumReTestHIVNeg_V1>'
AM_Shiny90NumRetestPLHIVOnART_Tag           = '<AM_Shiny90NumRetestPLHIVOnART_V1>'
AM_Shiny90NumRetestPLHIVNoART_Tag           = '<AM_Shiny90NumRetestPLHIVNoART_V1>'
AM_Shiny90NumNewDiagnoses_Tag               = '<AM_Shiny90NumNewDiagnoses_V1>'
AM_Shiny90PosRate_Tag                       = '<AM_Shiny90PosRate_V1>'
AM_Shiny90YieldNewDiagn_Tag                 = '<AM_Shiny90YieldNewDiagn_V1>'
AM_Shiny90IsFittedTag                       = '<AM_Shiny90IsFitted_V1>'
AM_KnowledgeOfStatusInputTypeTag            = '<AM_KnowledgeOfStatusInputType_V1>'
AM_KnowledgeOfStatusInputTag                = '<AM_KnowledgeOfStatusInput_V1>'
AM_KOSNewDiagnosesChildren0t14Tag           = '<AM_KOSNewDiagnosesChildren0t14_V1>'
AM_KOSNewDiagnosesAdults15PlusTag           = '<AM_KOSNewDiagnosesAdults15Plus_V1>'
AM_KOSNewDiagnosesRecCD4TestTag             = '<AM_KOSNewDiagnosesRecCD4Test_V1>'
        
AM_ViralSuppressionInputTypeTag             = '<AM_ViralSuppressionInputType_V1>'
AM_ViralSuppressionInputTag                 = '<AM_ViralSuppressionInput_V1>'
AM_ViralSuppressionThresholdTag             = '<AM_ViralSuppressionThreshold_V1>'
AM_Meningitis_Tag                           = '<AM_Meningitis_V1>'
AM_AdvOptsMeningitis_Tag                    = '<AM_AdvOptsMeningitis_V1>'

AM_ChildAnnRateProgressLowerCD4Tag          = '<AM_ChildAnnRateProgressLowerCD4_V1>'
AM_ChildDistNewInfectionsCD4Tag             = '<AM_ChildDistNewInfectionsCD4_V1>'
AM_ChildMortByCD4NoARTTag                   = '<AM_ChildMortByCD4NoART_V1>'
AM_ChildMortalityRatesTag                   = '<AM_ChildMortalityRates_V1>'
AM_ChildMortalityRates_MetaTag              = '<AM_ChildMortalityRates_Meta_V1>'
AM_ChildMortalityRatesMultiplierTag         = '<AM_ChildMortalityRatesMultiplier_V1>'
AM_ChildMortByCD4WithART0to6Tag             = '<AM_ChildMortByCD4WithART0to6_V1>'
AM_ChildMortByCD4WithART0to6_MetaTag        = '<AM_ChildMortByCD4WithART0to6_Meta_V1>'
AM_ChildMortByCD4WithART0to6PercTag         = '<AM_ChildMortByCD4WithART0to6Perc_V1>'
AM_ChildMortByCD4WithART0to6Perc_MetaTag    = '<AM_ChildMortByCD4WithART0to6Perc_Meta_V1>'
AM_ChildMortByCD4WithART7to12Tag            = '<AM_ChildMortByCD4WithART7to12_V1>'
AM_ChildMortByCD4WithART7to12_MetaTag       = '<AM_ChildMortByCD4WithART7to12_Meta_V1>'
AM_ChildMortByCD4WithART7to12PercTag        = '<AM_ChildMortByCD4WithART7to12Perc_V1>'
AM_ChildMortByCD4WithART7to12Perc_MetaTag   = '<AM_ChildMortByCD4WithART7to12Perc_Meta_V1>'
AM_ChildMortByCD4WithARTGT12Tag             = '<AM_ChildMortByCD4WithARTGT12_V1>'
AM_ChildMortByCD4WithARTGT12_MetaTag        = '<AM_ChildMortByCD4WithARTGT12_Meta_V1>'
AM_ChildMortByCD4WithARTGT12PercTag         = '<AM_ChildMortByCD4WithARTGT12Perc_V1>'
AM_ChildMortByCD4WithARTGT12Perc_MetaTag    = '<AM_ChildMortByCD4WithARTGT12Perc_Meta_V1>'
AM_ChildARTDistTag                          = '<AM_ChildARTDist_V1>'
AM_ChildARTDist_MetaTag                     = '<AM_ChildARTDist_Meta_V1>'
AM_EffectTreatChildTag                      = '<AM_EffectTreatChild_V1>'
AM_ChildWeightBandsTag                      = '<AM_ChildWeightBands_V1>'
AM_AdultAnnRateProgressLowerCD4Tag          = '<AM_AdultAnnRateProgressLowerCD4_V1>'
AM_AdultAnnRateProgressLowerCD4_MetaTag     = '<AM_AdultAnnRateProgressLowerCD4_Meta_V1>'
AM_AdultDistNewInfectionsCD4Tag             = '<AM_AdultDistNewInfectionsCD4_V1>'
AM_AdultDistNewInfectionsCD4_MetaTag        = '<AM_AdultDistNewInfectionsCD4_Meta_V1>'
AM_AdultMortByCD4NoARTTag                   = '<AM_AdultMortByCD4NoART_V1>'
AM_AdultMortByCD4NoART_MetaTag              = '<AM_AdultMortByCD4NoART_Meta_V1>'
AM_AdultInfectReducTag                      = '<AM_AdultInfectReduc_V1>'
AM_MortalityRatesTag                        = '<AM_MortalityRates_V1>'
AM_MortalityRates_MetaTag                   = '<AM_MortalityRates_Meta_V1>'
AM_MortalityRatesMultiplierTag              = '<AM_MortalityRatesMultiplier_V1>'
AM_AdultMortByCD4WithART0to6Tag             = '<AM_AdultMortByCD4WithART0to6_V1>'
AM_AdultMortByCD4WithART0to6_MetaTag        = '<AM_AdultMortByCD4WithART0to6_Meta_V1>'
AM_AdultMortByCD4WithART7to12Tag            = '<AM_AdultMortByCD4WithART7to12_V1>'
AM_AdultMortByCD4WithART7to12_MetaTag       = '<AM_AdultMortByCD4WithART7to12_Meta_V1>'
AM_AdultMortByCD4WithARTGT12Tag             = '<AM_AdultMortByCD4WithARTGT12_V1>'
AM_AdultMortByCD4WithARTGT12_MetaTag        = '<AM_AdultMortByCD4WithARTGT12_Meta_V1>'
AM_AdultNonAIDSExcessMortTag                = '<AM_AdultNonAIDSExcessMort_V2>'
AM_AdultNonAIDSExcessMort_Meta_Tag          = '<AM_AdultNonAIDSExcessMort_Meta_V1>'
AM_AdultNonAIDSExcessMortRegionTag          = '<AM_AdultNonAIDSExcessMortRegion_V1>'
AM_AdultNonAIDSExcessMortCustomFlagTag      = '<AM_AdultNonAIDSExcessMortCustomFlag_V1>'
AM_NonAIDSExcessDeathsSingleAgeTag          = '<AM_NonAIDSExcessDeathsSingleAge_V2>'

AM_TFRRegionTag                             = '<AM_TFRRegion_V1>'
AM_TFRRegion_MetaTag                        = '<AM_TFRRegion_Meta_V1>'
AM_HIVTFRCustomFlagTag                      = '<AM_HIVTFRCustomFlag_V1>'
AM_HIVTFRTag                                = '<AM_HIVTFR_V1>'
AM_HIVTFR_MetaTag                           = '<AM_HIVTFR_Meta_V1>'
AM_TFRInputTypeTag                          = '<AM_TFRInputType_V1>'
AM_FertCD4DiscountTag                       = '<AM_FertCD4Discount_V1>'
AM_FertCD4Discount_MetaTag                  = '<AM_FertCD4Discount_Meta_V1>'
AM_RatioWomenOnARTTag                       = '<AM_RatioWomenOnART_V1>'
AM_RatioWomenOnART_MetaTag                  = '<AM_RatioWomenOnART_Meta_V1>'
AM_FRRFitInputTag                           = '<AM_FRRFitInput_V1>'
AM_FRRbyLocationTag                         = '<AM_FRRbyLocation_V1>'
AM_TransEffAssumpTag                        = '<AM_TransEffAssump_V1>'
AM_DALYDisabilityWeightsTag                 = '<AM_DALYDisabilityWeights_V1>'
AM_RiskPopOrphansTag                        = '<AM_RiskPopOrphans_V1>'
AM_ECDCValuesTag                            = '<AM_ECDCValues_V1>'
AM_NosocomialInfectionsByAgeTag             = '<AM_NosocomialInfectionsByAge_V1>'
AM_HIVMigrantsBySexAgeTag                   = '<AM_HIVMigrantsBySexAge_V1>'
AM_HIVMigrantsBySingleAgeTag                = '<AM_HIVMigrantsBySingleAge_V1>'
AM_IncidenceInput1970Tag                    = '<AM_IncidenceInput1970_V1>'
AM_CSAVRInputAIDSDeathsTag                  = '<AM_CSAVRInputAIDSDeaths_V2>'
AM_CSAVRInputAIDSDeathsBySexTag             = '<AM_CSAVRInputAIDSDeathsBySex_V2>'
AM_CSAVRInputAIDSDeathsBySexAgeTag          = '<AM_CSAVRInputAIDSDeathsBySexAge_V2>'
AM_CSAVRInputAIDSDeathsSourceTag            = '<AM_CSAVRInputAIDSDeathsSource_V1>'
AM_CSAVRInputAIDSDeathsSourceNameTag        = '<AM_CSAVRInputAIDSDeathsSourceName_V1>'
AM_CSAVRInputNewDiagnosesTag                = '<AM_CSAVRInputNewDiagnoses_V1>'
AM_CSAVRInputNewDiagnosesBySexTag           = '<AM_CSAVRInputNewDiagnosesBySex_V1>'
AM_CSAVRInputNewDiagnosesBySexAgeTag        = '<AM_CSAVRInputNewDiagnosesBySexAge_V1>'
AM_CSAVRInputNewDiagnosesBySexAgeCD4Tag     = '<AM_CSAVRInputNewDiagnosesBySexAgeCD4_V1>'
AM_CSAVRInputCD4DistAtDiagTag               = '<AM_CSAVRInputCD4DistAtDiag_V1>'
AM_CSAVRPopulationValueTag                  = '<AM_CSAVRPopulationValue_V1>'
AM_AIDSMortalityAllAgesTag                  = '<AM_AIDSMortalityAllAges_V1>'
AM_AnnualInterruptionRateTag                = '<AM_AnnualInterruptionRate_V1>'
AM_IncreasedLikelihoodOfReinitTag           = '<AM_IncreasedLikelihoodOfReinit_V1>'
AM_OffARTMortRateMultiplierTag              = '<AM_OffARTMortRateMultiplier_V1>'
AM_ChildAnnualInterruptionRateTag           = '<AM_ChildAnnualInterruptionRate_V1>'
AM_ChildIncreasedLikelihoodOfReinitTag      = '<AM_ChildIncreasedLikelihoodOfReinit_V1>'
AM_ChildOffARTMortRateMultiplierTag         = '<AM_ChildOffARTMortRateMultiplier_V1>'
AM_CSAVRFitOptionsTag                       = '<AM_CSAVRFitOptions_V1>'
AM_CSAVRParametersTag                       = '<AM_CSAVRParameters_V1>'
AM_CSAVRIncScaleParametersTag               = '<AM_CSAVRIncScaleParameters_V1>'
AM_CSAVRUncertaintyParamsTag                = '<AM_CSAVRUncertaintyParams_V1>'
AM_CSAVRConstrainPLHIVGTNumARTTag           = '<AM_CSAVRConstrainPLHIVGTNumART_V1>'
AM_CSAVRTypeOfFitTag                        = '<AM_CSAVRTypeOfFit_V1>'
AM_CSAVRAdjustIRRsTag                       = '<AM_CSAVRAdjustIRRs_V2>'
AM_CSAVRModelFitTypesTag                    = '<AM_CSAVRModelFitTypes_V1>'
AM_CSAVRMetaDataTag                         = '<AM_CSAVRMetaData_V3>'
AM_CSAVRMeanCD4atDiagnosisTag               = '<AM_CSAVRMeanCD4atDiagnosis_V2>'
AM_CSAVRMeanCD4atDiagnosisByAgeSexTag       = '<AM_CSAVRMeanCD4atDiagnosisByAgeSex_V2>'
AM_CSAVRTimeInfToDiagTag                    = '<AM_CSAVRTimeInfToDiag_V2>'
AM_CSAVRPropOfDiagnosedTag                  = '<AM_CSAVRPropOfDiagnosed_V2>'
AM_CSAVRPropOfDiagnosedNoARTTag             = '<AM_CSAVRPropOfDiagnosedNoART_V2>'
AM_CSAVRPropOfDiagnosedByAgeSexCD4Tag       = '<AM_CSAVRPropOfDiagnosedByAgeSexCD4_V2>'
AM_CSAVRNumPLHIVTag                         = '<AM_CSAVRNumPLHIV_V2>'
AM_CSAVRNumPLHIVByAgeSexCD4Tag              = '<AM_CSAVRNumPLHIVByAgeSexCD4_V2>'
AM_CSAVRNumDiagnosedTag                     = '<AM_CSAVRNumDiagnosed_V2>'
AM_CSAVRNumDiagnosedByAgeSexCD4Tag          = '<AM_CSAVRNumDiagnosedByAgeSexCD4_V2>'
AM_CSAVRAIDSDeathsTag                       = '<AM_CSAVRAIDSDeaths_V2>'
AM_CSAVRAIDSDeathsByAgeSexTag               = '<AM_CSAVRAIDSDeathsByAgeSex_V2>'
AM_CSAVRNumNewInfectionsTag                 = '<AM_CSAVRNumNewInfections_V2>'
AM_CSAVRIncidenceByFitTag                   = '<AM_CSAVRIncidenceByFit_V1>'
AM_HIVSexRatioTag                           = '<AM_HIVSexRatio_V1>'
AM_HIVSexRatio_MetaTag                      = '<AM_HIVSexRatio_Meta_V3>'
AM_CSAVRHIVSexRatioTag                      = '<AM_CSAVRHIVSexRatio_V2>'
AM_HIVSexRatioFittingValuesTag              = '<AM_HIVSexRatioFittingValues_V1>'
AM_CustomSAPDataTag                         = '<AM_CustomSAPData_V2>'
AM_CustomSAPData_MetaTag                    = '<AM_CustomSAPData_Meta_V1>'
AM_CustomSAPDataIndexTag                    = '<AM_CustomSAPDataIndex_V2>'
AM_ARTTreatFitDistOfHIVTag                  = '<AM_ARTTreatFitDistOfHIV_V1>'
AM_ARTTreatFitHIVSexRatioTag                = '<AM_ARTTreatFitHIVSexRatio_V1>'
AM_ARTTreatFitHIVSexRatio_MetaTag           = '<AM_ARTTreatFitHIVSexRatio_Meta_V1>'
AM_FittingDataTag                           = '<AM_FittingData_V1>'
AM_ARTTreatFittingDataTag                   = '<AM_ARTTreatFittingData_V1>'
AM_DistOfHIVTag                             = '<AM_DistOfHIV_V1>'
AM_DistOfHIV_MetaTag                        = '<AM_DistOfHIV_Meta_V2>'
AM_CSAVRDistOfHIVTag                        = '<AM_CSAVRDistOfHIV_V3>'
AM_EPPANCRTDataTag                          = '<AM_EPPANCRTData_V1>'
AM_SAPFormDHSFitSurveyTag                   = '<AM_SAPFormDHSFitSurvey_V1>'
AM_IRRFittingValuesTag                      = '<AM_IRRFittingValues_V1>'
AM_AIDS45q15Tag                             = '<AM_AIDS45q15_V1>'
AM_NonAIDS45q15Tag                          = '<AM_NonAIDS45q15_V1>'
AM_Total45q15Tag                            = '<AM_Total45q15_V1>'
AM_Total45q15_MetaTag                       = '<AM_Total45q15_Meta_V1>'
AM_Under5MortRateTag                        = '<AM_Under5MortRate_V1>'
AM_PMTCTProgEstNeedTag                      = '<AM_PMTCTProgEstNeed_V1>'
AM_NumberOnARTTag                           = '<AM_NumberOnART_V1>'
AM_ARTCovByAgeTag                           = '<AM_ARTCovByAge_V1>'
AM_KeyPopsTag                               = '<AM_KeyPops_V1>'
AM_PregWomenPrevRoutineTestTag              = '<AM_PregWomenPrevRoutineTest_V1>'
AM_PregWomenPrevTag                         = '<AM_PregWomenPrev_V1>'
AM_PregWomenPrev_MetaTag                    = '<AM_PregWomenPrev_Meta_V1>'
AM_PrevSurveyTag                            = '<AM_PrevSurvey_V1>'
AM_PrevSurvey_MetaTag                       = '<AM_PrevSurvey_Meta_V1>'
AM_ARTCovSurveyTag                          = '<AM_ARTCovSurvey_V1>'
AM_ARTCovSurvey_MetaTag                     = '<AM_ARTCovSurvey_Meta_V1>'
AM_MortRateByAgeTag                         = '<AM_MortRateByAge_V1>'
AM_AllCauseMortalityTag                     = '<AM_AllCauseMortality_V1>'
AM_AllCauseMortality_MetaTag                = '<AM_AllCauseMortality_Meta_V1>'
AM_AIDSMortalityTag                         = '<AM_AIDSMortality_V1>'
AM_AIDSMortality_MetaTag                    = '<AM_AIDSMortality_Meta_V1>'
AM_NumberOnARTByAgeTag                      = '<AM_NumberOnARTByAge_V1>'
AM_NewlyStartingARTTag                      = '<AM_NewlyStartingART_V1>'
AM_AdultsChildrenStartingARTTag             = '<AM_AdultsChildrenStartingART_V1>'
AM_PercentOfPopTag                          = '<AM_PercentOfPop_V1>'
# AM_FutureARTTag                             = '<AM_FutureART_V1>'
AM_ARTCoverageSelectionTag                  = '<AM_ARTCoverageSelection_V1>'
AM_BFYearsRGIdxTag                          = '<AM_BFYearsRGIdx_V1>'
AM_BFArvRGIdxTag                            = '<AM_BFArvRGIdx_V1>'
AM_ChildHIVMortARTRegionTag                 = '<AM_ChildHIVMortARTRegion_V1>'
AM_ChildHIVMortARTCustomFlagTag             = '<AM_ChildHIVMortARTCustomFlag_V1>'
AM_ChildARTDistRegionTag                    = '<AM_ChildARTDistRegion_V1>'
AM_ChildARTDistCustomFlagTag                = '<AM_ChildARTDistCustomFlag_V1>'
AM_AdultProgressRatesRegionTag              = '<AM_AdultProgressRatesRegion_V1>'
AM_AdultProgressRatesCustomFlagTag          = '<AM_AdultProgressRatesCustomFlag_V1>'
AM_AdultHIVMortNoARTRegionTag               = '<AM_AdultHIVMortNoARTRegion_V1>'
AM_AdultHIVMortNoARTCustomFlagTag           = '<AM_AdultHIVMortNoARTCustomFlag_V1>'
AM_AdultHIVMortARTRegionTag                 = '<AM_AdultHIVMortARTRegion_V1>'
AM_AdultHIVMortARTCustomFlagTag             = '<AM_AdultHIVMortARTCustomFlag_V1>'
AM_DALYBaseYearTag                          = '<AM_DALYBaseYear_V1>'
AM_DALYDiscountRateTag                      = '<AM_DALYDiscountRate_V1>'
AM_DALYUseStandardLifeTableTag              = '<AM_DALYUseStandardLifeTable_V1>'
# AM_DALYTBWeightTag                          = '<AM_DALYTBWeight_V1>'
AM_OrphansRegionalPatternTag                = '<AM_OrphansRegionalPattern_V1>'
AM_IncidenceInput1970BoolTag                = '<AM_IncidenceInput1970Bool_V1>'
AM_IncidenceOptionsTag                      = '<AM_IncidenceOptions_V1>'
AM_IncidenceByFitTag                        = '<AM_IncidenceByFit_V1>'
AM_FourDecPlaceIDTag                        = '<AM_FourDecPlaceID_V1>'
AM_EPPPrevAdjTag                            = '<AM_EPPPrevAdj_V1>'
AM_EPPMaxAdjFactorTag                       = '<AM_EPPMaxAdjFactor_V1>'
# AM_EPPPrevAdjustmentYearTag                 = '<AM_EPPPrevAdjustmentYear_V1>'
AM_EPPPopulationAgesTag                     = '<AM_EPPPopulationAges_V1>'
# AM_IncEpidemicRGIdxTag                      = '<AM_IncEpidemicRGIdx_V1>'
AM_IncEpidemicCustomFlagTag                 = '<AM_IncEpidemicCustomFlag_V1>'
AM_SexRatioFromEPPTag                       = '<AM_SexRatioFromEPP_V1>'
# AM_SurveyYearIndexTag                       = '<AM_SurveyYearIndex_V1>'
AM_HIVSexRatioFromEPPTag                    = '<AM_HIVSexRatioFromEPP_V1>'
AM_EpidemicTypeFromEPPTag                   = '<AM_EpidemicTypeFromEPP_V1>'
AM_ChangesLogTag                            = '<AM_ChangesLog_V1>'
AM_AdultPrevalenceTag                       = '<AM_AdultPrevalence_V1>'
AM_EPPCountryNameTag                        = '<AM_EPPCountryName_V1>'
AM_EPPEpiNameTag                            = '<AM_EPPEpiName_V1>'
AM_EPPEpidemicTag                           = '<AM_EPPEpidemic_V2>'
AM_EPPPrevDataTag                           = '<AM_EPPPrevData_V1>'
AM_EPPIncDataTag                            = '<AM_EPPIncData_V1>'
AM_EPPPopDataTag                            = '<AM_EPPPopData_V1>'
AM_EPPSexRatioTag                           = '<AM_EPPSexRatio_V1>'
AM_EPPBaseYrPopTag                          = '<AM_EPPBaseYrPop_V1>'
AM_EPPIDUMortalityTag                       = '<AM_EPPIDUMortality_V1>'
AM_EPPPathInfoTag                           = '<AM_EPPPathInfo_V1>'
AM_EPPAgeRangeTag                           = '<AM_EPPAgeRange_V1>'
AM_NumOfEPPEpidemicsTag                     = '<AM_NumOfEPPEpidemics_V1>'
AM_YrPtPrevalence_WBTag                     = '<AM_YrPtPrevalence_WB_V2>'
AM_AIDSDeathsAmongIDUTag                    = '<AM_AIDSDeathsAmongIDU_V1>'
AM_PropIDU_WBTag                            = '<AM_PropIDU_WB_V1>'
AM_NonAIDSDeathsAmongIDUTag                 = '<AM_NonAIDSDeathsAmongIDU_V1>'
# AM_CountryProjUsedBySubNatRegionTag         = '<AM_CountryProjUsedBySubNatRegion_V1>'
# AM_CountryProjBigPopTag                     = '<AM_CountryProjBigPop_V1>'
AM_ChAged14ByCD4CatTag                      = '<AM_ChAged14ByCD4Cat_V1>'
# AM_IMRTag                                   = '<AM_IMR_V1>'
# AM_CMRTag                                   = '<AM_CMR_V1>'
AM_NewHIVInfByRiskGroupTag                  = '<AM_NewHIVInfByRiskGroup_V2>'
# AM_HIVPopByRiskGroupTag                     = '<AM_HIVPopByRiskGroup_V1>'
AM_HIVExpUninfChildrenTag                   = '<AM_HIVExpUninfChildren_V1>'
AM_ARTExpUninfChildrenTag                   = '<AM_ARTExpUninfChildren_V1>'
# AM_PrevIncid15PlusTag                       = '<AM_PrevIncid15Plus_V1>'
AM_NewInfantInfectionsTag                   = '<AM_NewInfantInfections_V1>'
# AM_HIVPop1517Tag                            = '<AM_HIVPop1517_V1>'
# AM_ChildARTNeed1To4Tag                      = '<AM_ChildARTNeed1To4_V1>'
# AM_ARTNeedUnder2Tag                         = '<AM_ARTNeedUnder2_V1>'
AM_CD4DistributionTag                       = '<AM_CD4Distribution_V1>'
AM_CD4Distribution15_49Tag                  = '<AM_CD4Distribution15_49_V1>'
AM_CD4DistributionChildTag                  = '<AM_CD4DistributionChild_V1>'
# AM_HIVPos15YearOldsTag                      = '<AM_HIVPos15YearOlds_V1>'
AM_DeathsByAgeTag                           = '<AM_DeathsByAge_V1>'
AM_HIVBySingleAgeTag                        = '<AM_HIVBySingleAge_V1>'
# AM_AdultDeathsTag                           = '<AM_AdultDeaths_V1>'
AM_RegionalOutputTag                        = '<AM_RegionalOutput_V1>'
AM_OrphansTag                               = '<AM_Orphans_V1>'
AM_AgeLastBDayTag                           = '<AM_AgeLastBDay_V1>'
AM_PopAdjTag                                = '<AM_PopAdj_V1>'
AM_PopAdjAmountTag                          = '<AM_PopAdjAmount_V1>'
# AM_HIVTag                                   = '<AM_HIV_V1>'
# AM_NewInfectionsTag                         = '<AM_NewInfections_V1>'
# AM_AIDSDeathsTag                            = '<AM_AIDSDeaths_V1>'
# AM_AIDSDeathsARTTag                         = '<AM_AIDSDeathsART_V1>'
AM_AIDSDeathsARTSingleAgeTag                = '<AM_AIDSDeathsARTSingleAge_V1>'
# AM_AIDSDeathsNoARTTag                       = '<AM_AIDSDeathsNoART_V1>'
AM_AIDSDeathsNoARTSingleAgeTag              = '<AM_AIDSDeathsNoARTSingleAge_V1>'
AM_HIVPregWomenTag                          = '<AM_HIVPregWomen_V1>'
AM_HIVPregWomenCD4LT350Tag                  = '<AM_HIVPregWomenCD4LT350_V1>'
# AM_ChildARTCalcTag                          = '<AM_ChildARTCalc_V1>'
AM_ChildOnCotrimTag                         = '<AM_ChildOnCotrim_V1>'
AM_ChildNeedCotrimTag                       = '<AM_ChildNeedCotrim_V1>'
AM_NeedARTTag                               = '<AM_NeedART_V1>'
AM_UnmetNeedARTTag                          = '<AM_UnmetNeedART_V1>'
# AM_HIVPopUnder1Tag                          = '<AM_HIVPopUnder1_V1>'
AM_AIDSDeathsByAgeTag                       = '<AM_AIDSDeathsByAge_V1>'
AM_PerinatalTransmissionTag                 = '<AM_PerinatalTransmission_V1>'
AM_ChildCTXNeed1To4Tag                      = '<AM_ChildCTXNeed1To4_V1>'
AM_TotalOrphansTag                          = '<AM_TotalOrphans_V1>'
AM_AllAIDSOrphTag                           = '<AM_AllAIDSOrph_V1>'
AM_AllAIDSNewOrphTag                        = '<AM_AllAIDSNewOrph_V1>'
AM_NewlyNeedARTTag                          = '<AM_NewlyNeedART_V1>'
AM_ChildNeedPMTCTTag                        = '<AM_ChildNeedPMTCT_V1>'
AM_ChildOnPMTCTTag                          = '<AM_ChildOnPMTCT_V1>'
AM_MTCTRate6WksTag                          = '<AM_MTCTRate6Wks_V1>'
# AM_HIVPos50YearOldsTag                      = '<AM_HIVPos50YearOlds_V1>'
# AM_HIVPos50YearOldsARTTag                   = '<AM_HIVPos50YearOldsART_V1>'
# AM_OnARTTag                                 = '<AM_OnART_V1>'
AM_OnARTBySingleAgeTag                      = '<AM_OnARTBySingleAge_V1>'
AM_OnARTBySingleAge_ProgDataTag             = '<AM_OnARTBySingleAge_ProgData_V1>'
AM_NewInfectionsBySingleAgeTag              = '<AM_NewInfectionsBySingleAge_V1>'
AM_NewlyOnARTTag                            = '<AM_NewlyOnART_V1>'
# AM_ARTDropOutRateTag                        = '<AM_ARTDropOutRate_V1>'
# AM_OnSLTag                                  = '<AM_OnSL_V1>'
# AM_NeedARTDec31Tag                          = '<AM_NeedARTDec31_V1>'
AM_ViralSupprPhi_TempTag                    = '<AM_ViralSupprPhi_Temp_V1>'
AM_SexuallyActive15to19Tag                  = '<AM_SexuallyActive15to19_V1>'
AM_SexuallyActive15to19_MetaTag             = '<AM_SexuallyActive15to19_Meta_V1>'
AM_ReadFRR15_19Coefs_TempTag                = '<AM_ReadFRR15_19Coefs_Temp_V1>'
AM_TFR15_19DefaultDataTag                   = '<AM_TFR15_19DefaultData_V1>'
AM_FRRFitXMLDefaultDataTag                  = '<AM_FRRFitXMLDefaultData_V1>'
AM_HAARTBySexPerNumTag                      = '<AM_HAARTBySexPerNum_V1>'
AM_AdultARTAdjFactorTag                     = '<AM_AdultARTAdjFactor_V1>'
AM_AdultARTAdjFactorFlagTag                 = '<AM_AdultARTAdjFactorFlag_V1>'
AM_AdultPatsAllocToFromOtherRegionTag       = '<AM_AdultPatsAllocToFromOtherRegion_V1>'
# AM_AdultARTByMonthTag                       = '<AM_AdultARTByMonth_V1>'
# AM_AdultPercentLTFUByMonthTag               = '<AM_AdultPercentLTFUByMonth_V1>'
AM_KnowledgeOfStatusFileTitleTag            = '<AM_KnowledgeOfStatusFileTitle_V1>'
AM_CascadeActiveTag                         = '<AM_CascadeActive_V1>'
AM_PMTCTEffRegTag                           = '<AM_PMTCTEffReg_V1>'
AM_HIVPosBFWomen3MonthsTag                  = '<AM_HIVPosBFWomen3Months_V1>'
AM_HIVPosBFWomen12MonthsTag                 = '<AM_HIVPosBFWomen12Months_V1>'
# AM_ImpactCalculatedTag                      = '<AM_ImpactCalculated_V1>'
AM_AIDSDeathsNoIntervTag                    = '<AM_AIDSDeathsNoInterv_V1>'
AM_DeathsAvertARTTag                        = '<AM_DeathsAvertART_V1>'
AM_InfectAvertPMTCTTag                      = '<AM_InfectAvertPMTCT_V1>'
AM_LifeYearsGainedTag                       = '<AM_LifeYearsGained_V1>'
AM_ChildDeathsAvertARTTag                   = '<AM_ChildDeathsAvertART_V1>'
AM_DeathsAvertCotrimTag                     = '<AM_DeathsAvertCotrim_V1>'
AM_DeathsAvertPMTCTTag                      = '<AM_DeathsAvertPMTCT_V1>'
AM_HIVAgingRateTag                          = '<AM_HIVAgingRate_V1>'
AM_HIVChildNoARTByInfectionTag              = '<AM_HIVChildNoARTByInfection_V1>'
AM_TreatPercentTag                          = '<AM_TreatPercent_V1>'
# AM_AIDSBirthsTag                            = '<AM_AIDSBirths_V1>'
# AM_ARTCalcTag                               = '<AM_ARTCalc_V1>'
# AM_ARTCalc_15PlusTag                        = '<AM_ARTCalc_15Plus_V1>'
AM_MedianCD4Tag                             = '<AM_MedianCD4_V1>'
# AM_NumChildOnARTUnder1Tag                   = '<AM_NumChildOnARTUnder1_V1>'
# AM_NumChildOnARTByAgeTag                    = '<AM_NumChildOnARTByAge_V1>'
AM_AIDSMortAllAgesFYrAdjIdxTag              = '<AM_AIDSMortAllAgesFYrAdjIdx_V1>'
AM_ValidationNumberOnARTByAgeTag            = '<AM_ValidationNumberOnARTByAge_V1>'
AM_ValidationNewlyOnARTTag                  = '<AM_ValidationNewlyOnART_V1>'
AM_ValidationMortalityRateByAgeTag          = '<AM_ValidationMortalityRateByAge_V1>'
AM_ValidationAIDSMortalityByAgeTag          = '<AM_ValidationAIDSMortalityByAge_V1>'
AM_ValidationAllCausesMortalityByAgeTag     = '<AM_ValidationAllCausesMortalityByAge_V1>'
AM_ValidationPrevalenceSelectionTag         = '<AM_ValidationPrevalenceSelection_V1>'
AM_ValidationARTCoverageSelectionTag        = '<AM_ValidationARTCoverageSelection_V1>'
AM_ValidationAllCauseDeathsARTTag           = '<AM_ValidationAllCauseDeathsART_V1>'
AM_ValidationWaterfallTag                   = '<AM_ValidationWaterfall_V1>'
AM_PercART50PlusTag                         = '<AM_PercART50Plus_V2>'
AM_Prop350_500Tag                           = '<AM_Prop350_500_V1>'
AM_IncidenceAdjustmentFactorTag             = '<AM_IncidenceAdjustmentFactor_V1>'
AM_ECDCFQNameTag                            = '<AM_ECDCFQName_V1>'
AM_NewARTPatAllocTag                        = '<AM_NewARTPatAlloc_V1>'
AM_NewARTPatAlloc_MetaTag                   = '<AM_NewARTPatAlloc_Meta_V1>'
AM_NewARTPatAllocMethodTag                  = '<AM_NewARTPatAllocMethod_V1>'
AM_DefaultEpidemicTag                       = '<AM_DefaultEpidemic_V1>'
AM_FirstYearOfEpidemicTag                   = '<AM_FirstYearOfEpidemic_V1>'
# AM_DefaultDataCheckTag                      = '<AM_DefaultDataCheck_V1>'
AM_ResNonAIDSDeathsHIVPopNoARTTag           = '<AM_ResNonAIDSDeathsHIVPopNoART_V1>'
AM_ResMTCTBySourceTag                       = '<AM_ResMTCTBySource_V1>'
AM_ResDetailedTreatmentCascadeTag           = '<AM_ResDetailedTreatmentCascade_V2>'
AM_ResTreatmentCascadeTag                   = '<AM_ResTreatmentCascade_V2>'
AM_ResPerinatallyInf15_19Tag                = '<AM_ResPerinatallyInf15_19_V1>'
AM_ResPerinatallyInf20_24Tag                = '<AM_ResPerinatallyInf20_24_V1>'
AM_ResQALYsTag                              = '<AM_ResQALYs_V1>'
AM_ResDALYsAIDSOnlyTag                      = '<AM_ResDALYsAIDSOnly_V1>'
AM_ResDALYsTag                              = '<AM_ResDALYs_V1>'
AM_ResYLDTag                                = '<AM_ResYLD_V1>'
AM_ResNonAIDSDeathsHIVPopARTTag             = '<AM_ResNonAIDSDeathsHIVPopART_V1>'
AM_ResTotalDeathsHIVPopTag                  = '<AM_ResTotalDeathsHIVPop_V1>'
AM_ResAIDS45q15Tag                          = '<AM_ResAIDS45q15_V1>'
AM_ResNonAIDS45q15Tag                       = '<AM_ResNonAIDS45q15_V1>'
AM_ResNumWithAHDTag                         = '<AM_ResNumWithAHD_V1>'     
AM_ResNumWithAHDOnARTTag                    = '<AM_ResNumWithAHDOnART_V1>'
AM_ResNumWithAHDNoARTTag                    = '<AM_ResNumWithAHDNoART_V1>'
AM_Res15PlusNumWithCryptoAntigenemiaTag     = '<AM_Res15PlusNumWithCryptoAntigenemia_V1>'
AM_Res15PlusNumWithCryptoMeningitisTag      = '<AM_Res15PlusNumWithCryptoMeningitis_V1>'
AM_Res15PlusDeathsToPLHIVFromCMTag          = '<AM_Res15PlusDeathsToPLHIVFromCM_V1>'
AM_Res15PlusPercAIDSDeathsFromCMTag         = '<AM_Res15PlusPercAIDSDeathsFromCM_V1>'
AM_ResOnARTgt1Yr15PlusTag                   = '<AM_OnARTgt1Yr15Plus_V1>'
AM_ResOnARTlt1Yr15PlusTag                   = '<AM_OnARTlt1Yr15Plus_V1>'

AM_PreventionNeedsFirstTimeTag       = '<AM_PreventionNeeds_FirstTime_V1>'
AM_PreventionNeedsShowVMMCTabTag     = '<AM_PreventionNeeds_ShowVMMCTab_V1>'
AM_PreventionNeedsCondomsTag         = '<AM_PreventionNeeds_Condoms_V1>'
AM_PreventionNeedsVMMCTag            = '<AM_PreventionNeeds_VMMC_V1>'
AM_PreventionNeedsKeyPopsTag         = '<AM_PreventionNeeds_KeyPops_V1>'
AM_PreventionNeedsPrEPTag            = '<AM_PreventionNeeds_PrEP_V3>'

# AM_PrEPParametersTag                  = '<AM_PrEPParameters_V1>'


AM_ChildARTAdjFactorTag                     = '<AM_ChildARTAdjFactor_V1>'
AM_ChildARTAdjFactorFlagTag                 = '<AM_ChildARTAdjFactorFlag_V1>'
AM_ChildPatsAllocToFromOtherRegionTag       = '<AM_ChildPatsAllocToFromOtherRegion_V1>'

AM_SubnatPrevalenceTag       = '<AM_SubnatPrevalence_V1>'
AM_SubnatARTCoverageTag      = '<AM_SubnatARTCoverage_V1>'
AM_SubnatCustomBaseYrFlagTag = '<AM_SubnatCustomBaseYrFlag_V1>'

AM_PMTCTPrEPParametersTag = '<AM_PMTCTPrEPParameters_V2>'
AM_PMTCTReceivingOralPrEPTag = '<AM_PMTCTReceivingOralPrEP_V1>'
AM_PMTCTReceivingInjectablePrEPTag = '<AM_PMTCTReceivingInjectablePrEP_V1>'



from os import environ
from SpectrumCommon.Const.GB import *
from AvenirCommon.Database import GB_get_db_json

def _get_language_strings_dict(lang_ID):
    language_strings_dict = {}

    if lang_ID == GB_ENGLISH_MST_ID:
        english_DB_file_name = GB_ENGLISH_STRINGS_DB_NAME + '_' + GB_STRING_DB_CURR_VERSION + '.' + GB_JSON
        language_strings_dict = GB_get_db_json(environ[GB_SPECT_MOD_DATA_CONN_ENV], GB_GB_CONTAINER, english_DB_file_name)

    elif lang_ID == GB_SPANISH_MST_ID:
        spanish_DB_file_name = GB_SPANISH_STRINGS_DB_NAME + '_' + GB_STRING_DB_CURR_VERSION + '.' + GB_JSON
        language_strings_dict = GB_get_db_json(environ[GB_SPECT_MOD_DATA_CONN_ENV], GB_GB_CONTAINER, spanish_DB_file_name)

    elif lang_ID == GB_FRENCH_MST_ID:
        french_DB_file_name = GB_FRENCH_STRINGS_DB_NAME + '_' + GB_STRING_DB_CURR_VERSION + '.' + GB_JSON
        language_strings_dict = GB_get_db_json(environ[GB_SPECT_MOD_DATA_CONN_ENV], GB_GB_CONTAINER, french_DB_file_name)

    elif lang_ID == GB_ARABIC_MST_ID:
        pass

    elif lang_ID == GB_RUSSIAN_MST_ID:
        russian_DB_file_name = GB_RUSSIAN_STRINGS_DB_NAME + '_' + GB_STRING_DB_CURR_VERSION + '.' + GB_JSON
        language_strings_dict = GB_get_db_json(environ[GB_SPECT_MOD_DATA_CONN_ENV], GB_GB_CONTAINER, russian_DB_file_name)

    elif lang_ID == GB_PORTUGUESE_MST_ID:
        pass

    elif lang_ID == GB_CHINESE_MST_ID:
        pass

    elif lang_ID == GB_INDONESIAN_MST_ID:
        pass

    elif lang_ID == GB_UKRANIAN_MST_ID:
        ukranian_DB_file_name = GB_UKRANIAN_STRINGS_DB_NAME + '_' + GB_STRING_DB_CURR_VERSION + '.' + GB_JSON
        language_strings_dict = GB_get_db_json(environ[GB_SPECT_MOD_DATA_CONN_ENV], GB_GB_CONTAINER, ukranian_DB_file_name)

    return language_strings_dict

def _get_strings(lang_ID):
    enStrings = _get_language_strings_dict(GB_ENGLISH_MST_ID)
    langStrings = _get_language_strings_dict(lang_ID)
    enStrings.update(langStrings)
    return enStrings
